;
;	Part 12 Event Handler Routines - Flat, Dark Correct to Science Images
;
;-----------------------------------------------------------------
;
pro P12_Select_Night, Event
	;
	MSP_BDP_print_text, Event, "Night Selection Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	;
	wid = (*s_ptr).P12_droplist_night_wid[0]
	;
	night_index = Event.index
	;
	;	index is not enough, as we might not have done all the part 2
	;	stuff.... So, need to get the night designation from the list
	;
	WIDGET_CONTROL, wid, GET_VALUE = vector_night_desig
	selected_night_desig = vector_night_desig[night_index]
	;
	night_desig = selected_night_desig
	night_text = STRING(format='(I8.8)',night_desig)
	;
	;	reset buttons, and LEDs
	;
	for iw = 0, N_ELEMENTS( (*s_ptr).P12_button_wid)-1 do begin
		if(iw le 0) then sense = 1 else sense = 0
		P12_button_sensitize, Event, iw, sense
		;if(iw le 1) then P5_base_sensitize, Event, iw, 0
		if(iw eq 0) then P12_LED_sensitize, Event, iw, 0
	endfor
	;
	;	read in that night's save file
	;
	MSP_BDP_print_text, Event, "Opening Night Summary for night = "+night_text
	;
	MSP_BDP_Create_Night_File_Path, Event, night_desig, night_file_path
	MSP_BDP_Restore_Night_File, Event, night_file_path, description, ierr
	;
	if(ierr ne 0) then begin
		MSP_BDP_print_text, Event, "Could not restore night "+night_text+" - skipping"
		goto, P12SN_end
	endif
	;
	;	must redo pointers
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	raw_root = (*s_ptr).root_raw
	;
	;	capture number of groups
	;
	ngroups = (*n_ptr).num_groups
	ngroups_text = STRING(format='(I)',ngroups)
	MSP_BDP_print_text, Event, "Restored "+night_text+" with "+ngroups_text+" groups"
	;
	;	turn on first buttons
	;
	P12_button_sensitize, Event, 0, 1
	P12_button_sensitize, Event, 3, 1
	;
	;	load ZHT correct checkbox
	;
	(*s_ptr).P12_Ignore_ZHT_flag = 1 - (*s_ptr).P12_Ignore_ZHT_flag
	P12_Toggle_ZHT_Correct_Checkbox, Event
	;
	P12SN_end:
end
;
;----------------------------------------------------------------
pro P12_Toggle_ZHT_Correct_Checkbox, Event
	;
	;	toggles the Ignore Zero Hop Tests checkbox and flag
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	;
	(*s_ptr).P12_Ignore_ZHT_flag = 1 - (*s_ptr).P12_Ignore_ZHT_flag
	WIDGET_CONTROL, (*s_ptr).P12_checkbox_wid, SET_BUTTON = (*s_ptr).P12_Ignore_ZHT_flag
	;
	if( (*s_ptr).P12_Ignore_ZHT_flag eq 1 ) then begin
		MSP_BDP_print_text, Event, "Setting mode to Ignore Zero Hop test results"
	endif else begin
		MSP_BDP_print_text, Event, "Setting mode to Obey Zero Hop test results"
	endelse
end
;
;-----------------------------------------------------------------
pro P12_Toggle_ZHT_Dither, Event
	;
	;	toggles the fix-by-dither checkbox and flag
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	;
	(*s_ptr).P12_zht_dither_flag = 1 - (*s_ptr).P12_zht_dither_flag
	WIDGET_CONTROL, (*s_ptr).P12_radio_dither_wid, SET_BUTTON = (*s_ptr).P12_zht_dither_flag
	;
	if( (*s_ptr).P12_zht_dither_flag ) then begin
		MSP_BDP_print_text, Event, "Setting mode to do ZH correction by dither set"
	endif else begin
		MSP_BDP_print_text, Event, "Setting mode to do any ZH correction on full observations, only"
	endelse
end
;
;---------------------------------------------------------------------
pro P12_Select_Groups, Event, obs_type
	;
	;	from the list of groups for the selected night, cull those
	;	that are either pol or img/spec
	;
	MSP_BDP_print_text, Event, "Select Groups Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	f_ptr = PTR_struct.PTR_FITS
	bdf_ptr = PTR_struct.PTR_BDF
	;
	n_groups = (*n_ptr).num_groups
	;
	index_list = lonarr(n_groups)
	good_groups = 0L
	;
	for ig = 0, n_groups-1 do begin
		;
		this_group_type = (*n_ptr).groups[ig].group_type
		;
		;	for now, skip groups with sub-framing
		;
		this_group_frame = (*n_ptr).groups[ig].group_frame
		if( STRUPCASE(this_group_frame) ne 'FULL') then goto, group_loop_end
		;
		if(obs_type eq 'pol') then begin
			if(this_group_type eq 'Pol-Spec' or this_group_type eq 'Pol-Img') then begin
				index_list[good_groups] = ig
				good_groups++
			endif
		endif else begin
			if(obs_type eq 'phot') then begin
				if(this_group_type eq 'Spec' or this_group_type eq 'S-Argon' or this_group_type eq 'Image') then begin
					index_list[good_groups] = ig
					good_groups++
				endif
			endif
		endelse
		group_loop_end:
	endfor
	;
	(*bdf_ptr).n_groups = good_groups
	if(good_groups ne 0) then begin
		(*bdf_ptr).group_index[0:good_groups-1] = index_list[0:good_groups-1]
		(*bdf_ptr).status[0:good_groups-1] = 'To Do'
		MSP_BDP_print_text, Event, "Found "+STRING(format='(I5.1)',good_groups)+' groups needing processing'
		;
		;**************************************
		;	do matching for BPM, dark, flat images here
		;**************************************
		;
		for i_good = 0, good_groups-1 do begin
			;
			i_group = index_list[i_good]
			;
			;	get all the group info from the already loaded structure
			;
			MSP_BDP_Query_Group_Info, Event, i_group, filter_name, this_obs_type, $
				camera_name, HWP_angle, ut, exptime, date_obs, image_jd
;			;
;			;	get the critical information about the first image in this
;			;	group, drawing the information from the LC files
;			;
;			image_0_night = STRTRIM(STRING(format='(i8.8)',(*n_ptr).night_desig),2)
;
;			image_0_dir = (*s_ptr).root_reduced + '8_Linearity_Corrected_Images' + PATH_SEP() + $
;							image_0_night + PATH_SEP()
;			;
;			image_0_image_number = (*n_ptr).groups[i_group].group_start
;
;			;
;			image_0_path = image_0_dir + image_0_night + '.' + STRTRIM(STRING(format='(i5.3)',image_0_image_number),2) + $
;							'_LC.fits'
;			;
;			image_header = HEADFITS(image_0_path, /silent)
;			n_header_lines = N_ELEMENTS(image_header)
;			(*s_ptr).raw_header[0:n_header_lines-1] = image_header
;			(*s_ptr).raw_header[n_header_lines:N_ELEMENTS((*s_ptr).raw_header)-1] = ''
;			;
;			MSP_BDP_Parse_FITS_Header, Event
;			;
;			MSP_BDP_Filter_Classify, Event, filter_name
;			MSP_BDP_Observation_Classify, Event, filter_name, this_obs_type
;			;
;			;	if the first image has a junky filter list, fix it
;			;
;			test_filt_bad = STRPOS(STRUPCASE(filter_name),"VALUE NOT") ne -1
;			if(test_filt_bad) then begin
;				;
;				;	get the filter type from the group list (presumably edited)
;				;
;				filter_type = (*n_ptr).groups[i_group].group_filter
;				;
;				Test_Fix_Bad_Filter_Names, Event, filter_type, PATHNAME=image_0_path
;				;
;				;	reevaluate
;				;
;				image_header = HEADFITS(image_0_path, /silent)
;				n_header_lines = N_ELEMENTS(image_header)
;				(*s_ptr).raw_header[0:n_header_lines-1] = image_header
;				(*s_ptr).raw_header[n_header_lines:N_ELEMENTS((*s_ptr).raw_header)-1] = ''
;				;
;				MSP_BDP_Parse_FITS_Header, Event
;				;
;				MSP_BDP_Filter_Classify, Event, filter_name
;				MSP_BDP_Observation_Classify, Event, filter_name, this_obs_type
;			endif
;			;
;			;	match this_obs_type to the type of flat expected
;			;
			case this_obs_type of
				'S-Argon': flat_obs_type = 'S-Flat'
				'Spec': flat_obs_type = 'S-Flat'
				;
				'Image': flat_obs_type = 'I-Flat'
				;
				'Pol-Img': flat_obs_type = 'P-Flat'
				;
				else:	flat_obs_type = 'None'			; do spec pol later
			endcase
			;
			;	find best BPM by minimizing JD difference
			;
			n_bpm = (*bdf_ptr).n_bpm
			d_jd = ABS(image_jd - (*bdf_ptr).bpm_jd[0:n_bpm-1])
			min_jd = MIN(d_jd)
			min_jd_ind = WHERE(d_jd eq min_jd, count)
			if(count ge 1) then begin
				(*bdf_ptr).group_bpm_index[i_good] = min_jd_ind[0]
				;this_bpm_filename = FILE_BASENAME((*bdf_ptr).bpm_path[min_jd_ind[0]])
				;
				;print, "closest JD = ",(*bdf_ptr).bpm_jd[min_jd_ind[0]], " for BPM file = ",$
			;			FILE_BASENAME( (*bdf_ptr).bpm_path[min_jd_ind[0]]), " gr index = ",min_jd_ind[0]
				;
			endif
			;
			;	next, pick Dark as correct exptime match, then minimum jd
			;
			n_dark = (*bdf_ptr).n_dark
			same_expt_ind = WHERE(exptime eq (*bdf_ptr).dark_exptime[0:n_dark-1],count)
			if(count lt 1) then begin
				MSP_BDP_print_text, Event, "ERROR - For Group "+STRING(format='(i5.1)',i_group)+$
						' there was no matching Dark exposure time. Skipping'
				(*bdf_ptr).group_dark_index[i_good] = -1
			endif else begin
				;
				;	got at least one matching time, now minimize JD
				;
				dd_jd = ABS(image_jd - (*bdf_ptr).dark_jd[same_expt_ind])
				dd_min_jd = MIN(dd_jd)
				min_dd_jd_ind = WHERE(dd_jd eq dd_min_jd, dd_count)
				if(dd_count ge 1) then begin
					index = same_expt_ind[min_dd_jd_ind[0]]
					(*bdf_ptr).group_dark_index[i_good] = index
				endif
			endelse
			;
			;	now, match flats to correct flat type
			;
			n_flat = (*bdf_ptr).n_flat
			n_polar_flat = (*bdf_ptr).n_polar_flat
			;
			;	for non-pol, match correct flat
			;
			if(obs_type eq 'phot') then begin
				;
				same_type_filter_camera_ind = WHERE((STRPOS( (*bdf_ptr).flat_type[0:n_flat-1], flat_obs_type) ne -1) and $
													( (*bdf_ptr).flat_filter[0:n_flat-1] eq filter_name) and $
													( (*bdf_ptr).flat_camera[0:n_flat-1] eq camera_name), count)
				if(count lt 1) then begin
					;
					;	check for PV camera case. this will use regular F5 or F17 flats
					;
					if(camera_name eq 'PV') then begin
						same_type_filter_ind = WHERE((STRPOS( (*bdf_ptr).flat_type[0:n_flat-1], flat_obs_type) ne -1) and $
													( (*bdf_ptr).flat_filter[0:n_flat-1] eq filter_name),  xcount)
						if(xcount ge 1) then begin
							df_jd = ABS(image_jd - (*bdf_ptr).flat_jd[same_type_filter_ind])
							df_min_jd = MIN(df_jd)
							min_df_jd_ind = WHERE(df_jd eq df_min_jd, df_count)
							if(df_count ge 1) then begin
								index = same_type_filter_ind[min_df_jd_ind[0]]
								(*bdf_ptr).group_flat_index[i_good] = index
							endif
						endif else begin
							MSP_BDP_print_text, Event, "ERROR - no appropriate flats found for PV images of group "+$
											STRING(format='(i)',i_group)
							print, "for no match of PV group ",i_group
							print, "   flat_obs_type = ",flat_obs_type
							print, "   filter_name = ",filter_name
							print, "   camera_name = ",camera_name
							(*bdf_ptr).group_flat_index[i_good] = -1
						endelse
					endif else begin
						;
						MSP_BDP_print_text, Event, "ERROR - For Group "+STRING(format='(i5.1)',i_group)+$
								' there was no matching Flat Field image. Skipping'
						;
						print, "for no match of group ",i_group
						print, "   flat_obs_type = ",flat_obs_type
						print, "   filter_name = ",filter_name
						print, "   camera_name = ",camera_name
						;
						(*bdf_ptr).group_flat_index[i_good] = -1
					endelse
				endif else begin
					;
					;	got at least one match, now minimize JD if type is 'S-Argon' or 'Image', else
					;	if 'Spec', then flat must have larger JD than image
					;
					if(this_obs_type ne 'Spec') then begin
						df_jd = ABS(image_jd - (*bdf_ptr).flat_jd[same_type_filter_camera_ind])
						df_min_jd = MIN(df_jd)
						min_df_jd_ind = WHERE(df_jd eq df_min_jd, df_count)
						if(df_count ge 1) then begin
							index = same_type_filter_camera_ind[min_df_jd_ind[0]]
							(*bdf_ptr).group_flat_index[i_good] = index
						endif
					endif else begin
						spec_jd_ind = WHERE((image_jd lt (*bdf_ptr).flat_jd[same_type_filter_camera_ind]), s_count)
						if(s_count ge 1) then begin
							ds_jd = ABS(image_jd - (*bdf_ptr).flat_jd[same_type_filter_camera_ind[spec_jd_ind]])
							df_min_jd = MIN(ds_jd)
							min_df_jd_ind = WHERE(ds_jd eq df_min_jd, df_count)
							if(df_count ge 1) then begin
								index = same_type_filter_camera_ind[spec_jd_ind[min_df_jd_ind[0]]]
								(*bdf_ptr).group_flat_index[i_good] = index
							endif
						endif else begin
							MSP_BDP_print_text, Event, "ERROR - no later flats found for spec image of group "+$
											STRING(format='(i)',i_group)
							(*bdf_ptr).group_flat_index[i_good] = -1
						endelse
					endelse
				endelse
			endif else begin
				;
				;print, "number of polar flats = ",n_polar_flat
				;
				;	polar groups need to be separated into individual images for flat processing with correct HWP angle,
				;	start by rejecting all non-pol groups, though
				;
				;	DPC - 20080529 - Polar groups need not have PA=0 deg image, though, due to bad images
				;					 or split groups
				;
				same_type_filter_camera_ind = WHERE((STRPOS( (*bdf_ptr).polar_flat_type[0:n_polar_flat-1], flat_obs_type) ne -1) and $
													( (*bdf_ptr).polar_flat_filter[0:n_polar_flat-1] eq filter_name) and $
													( (*bdf_ptr).polar_flat_camera[0:n_polar_flat-1] eq camera_name), count) ; and $
												;	( (*bdf_ptr).polar_flat_HWP[0:n_polar_flat-1] eq 0.00), count) ; ***
				if(count lt 1) then begin
					MSP_BDP_print_text, Event, "ERROR - For Group "+STRING(format='(i5.1)',i_group)+$
							' there was no matching Polar Flat Field image. Skipping'
					;
					print, "for no match of group ",i_group
					print, "   flat_obs_type = ",flat_obs_type
					print, "   filter_name = ",filter_name
					print, "   camera_name = ",camera_name
					;
					print, "structure arrays: polar_flat_filter = ",(*bdf_ptr).polar_flat_filter[0:n_polar_flat-1]
					print, "  polar_flat_camera = ",(*bdf_ptr).polar_flat_camera[0:n_polar_flat-1]
					print, "  polar_flat_type   = ",(*bdf_ptr).polar_flat_type[0:n_polar_flat-1]
					;
					(*bdf_ptr).group_polar_flat_index[i_good] = -1
				endif else begin
					;
;					print, "for match of group ",i_group
;					print, "   flat_obs_type = ",flat_obs_type
;					print, "   filter_name = ",filter_name
;					print, "   camera_name = ",camera_name
;					;
;					print, "structure arrays: polar_flat_filter = ",(*bdf_ptr).polar_flat_filter[0:n_polar_flat-1]
;					print, "  polar_flat_camera = ",(*bdf_ptr).polar_flat_camera[0:n_polar_flat-1]
;					print, "  polar_flat_type   = ",(*bdf_ptr).polar_flat_type[0:n_polar_flat-1]
;					print, "  polar_flat_HWP    = ",(*bdf_ptr).polar_flat_HWP[0:n_polar_flat-1]
					;
					;	got at least one match, now minimize JD (for HWP=0 case)
					;
					;	DPC - 20080529 - changed to minimum HWP angle, to survive case of no HWP=0 images
					;
					if(this_obs_type ne 'Pol-Spec') then begin
						df_jd = ABS(image_jd - (*bdf_ptr).polar_flat_jd[same_type_filter_camera_ind])
						df_min_jd = MIN(df_jd)
						min_df_jd_ind = WHERE(df_jd eq df_min_jd, df_count)
						if(df_count ge 1) then begin
							index = same_type_filter_camera_ind[min_df_jd_ind[0]]
							;
							;print, "same_type_filter_camera_ind = ",same_type_filter_camera_ind
							;
							;	which polar flat set is this?
							;
							i_pol_set = -1
							hwp_ind = 0
							for i_set = 0, (*bdf_ptr).n_polar_flat_sets-1 do begin
								if(index eq hwp_ind) then begin
									i_pol_set = i_set
								endif
								hwp_ind = hwp_ind + (*bdf_ptr).polar_flat_set_elements[i_set]
							endfor
							if(i_pol_set eq -1) then begin
								MSP_BDP_print_text, Event, "Error - failed to match polar flat sets via minimum HWP"
								goto, P10FG_end
							endif
							(*bdf_ptr).group_polar_flat_index[i_good] = i_pol_set
						endif
					endif else begin
;						spec_jd_ind = WHERE((image_jd lt (*bdf_ptr).polar_flat_jd[same_type_filter_camera_ind]), s_count)
;						if(s_count ge 1) then begin
;							ds_jd = ABS(image_jd - (*bdf_ptr).polar_flat_jd[same_type_filter_camera_ind[spec_jd_ind]])
;							df_min_jd = MIN(ds_jd)
;							min_df_jd_ind = WHERE(ds_jd eq df_min_jd, df_count)
;							if(df_count ge 1) then begin
;								index = same_type_filter_camera_ind[spec_jd_ind[min_df_jd_ind[0]]]
;								(*bdf_ptr).group_flat_index[i_good] = index
;							endif
;						endif else begin
;							MSP_BDP_print_text, Event, "ERROR - no later flats found for spec image of group "+$
;											STRING(format='(i)',i_group)
;							(*bdf_ptr).group_flat_index[i_good] = -1
;						endelse
					endelse
				endelse
			endelse
		endfor
		;
		if(obs_type eq 'phot') then P12_button_sensitize, Event, 1, 1
		if(obs_type eq 'pol') then P12_button_sensitize, Event, 4, 1
		;
	endif else begin
		MSP_BDP_print_text, Event, "No groups found that need processing. Skipping."
	endelse
	;
	P10FG_end:
	;
end
;
;-----------------------------------------------------------------
pro P12_View_Edit_Group_Table, Event, obs_type
	;
	;	show list of groups, allow selection/deselection
	;
	MSP_BDP_print_text, Event, "View-Edit Groups Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	bdf_ptr = PTR_struct.PTR_BDF
	;
	;	number of groups
	;
	n_groups = (*bdf_ptr).n_groups
	;
	;	indices
	;
	group_indices = (*bdf_ptr).group_index[0:n_groups-1]
	;
	;------------------------------------------------------------------------------------
	;
	;	build the table structure
	;
	col_headers = ['Group', '1st Img','ObsType', 'Filter', 'Exp_Time', 'Cam', 'BadPix Mask',$
				'_____Dark_____', '_____Flat_____', 'Status']
	ncols = N_ELEMENTS(col_headers)
	;
	n_lines = n_groups
	format = strarr(ncols,n_lines)
	format[0:ncols-1,*] = '(a)'
	;
	color_array = strarr(ncols,n_lines)
	;
	row_headers = strarr(n_lines)
	data_line = {group:'', image_0:'', type:'', filter:'', exptime:'', camera:'', BPM:'',$
	             dark:'', flat:'', status: ''}
	;
	data_array = REPLICATE(data_line, n_lines)			; array of structures
	;
	for i = 0, n_lines-1 do begin
			;
			gi = group_indices[i]
			;
			;	put group names in the row headers
			;
;			row_headers[i] = 'LDFC_'+STRTRIM(STRING(format='(i4.1)',i),2)
			row_headers[i] = (*n_ptr).groups[gi].group_object
			;
			data_array[i].group = STRING(format='("Gr_",I3.3)',gi)
			data_array[i].image_0 = STRING(format='(I5.3)',(*n_ptr).groups[gi].group_start)
			data_array[i].type = (*n_ptr).groups[gi].group_type
			data_array[i].filter = (*n_ptr).groups[gi].group_filter
			data_array[i].exptime = STRING(format='(f8.3)',(*n_ptr).groups[gi].group_exptime)
			data_array[i].camera = (*n_ptr).groups[gi].group_camera
			;
			;	use the saved indices to lookup the filenames for these guys, unless the index
			;	is -1, then write "None" and flag with red
			;
			bpm_index = (*bdf_ptr).group_bpm_index[i]
			data_array[i].BPM = FILE_BASENAME( (*bdf_ptr).bpm_path[bpm_index])
			;
			;print, "Group = ",i," BPM index = ",bpm_index," BPM file = ",data_array[i].BPM
			;
			dark_index = (*bdf_ptr).group_dark_index[i]
			if(dark_index eq -1) then begin
				data_array[i].dark = "None"
				(*bdf_ptr).status[i] = 'Error'
			endif else begin
				data_array[i].dark = FILE_BASENAME( (*bdf_ptr).dark_path[dark_index])
			endelse
			;
			if(obs_type eq 'phot') then begin
				flat_index = (*bdf_ptr).group_flat_index[i]
				if(flat_index eq -1) then begin
					data_array[i].flat = 'None'
					(*bdf_ptr).status[i] = 'Error'
				endif else begin
					data_array[i].flat = FILE_BASENAME( (*bdf_ptr).flat_path[flat_index])
				endelse
			endif
			;
			if(obs_type eq 'pol') then begin
				polar_flat_set = (*bdf_ptr).group_polar_flat_index[i]	; this is the polar set index
				if(polar_flat_set eq -1) then begin
					data_array[i].flat = 'None'
					(*bdf_ptr).status[i] = 'Error'
				endif else begin
					if(polar_flat_set eq 0) then polar_flat_index = 0 else begin
						polar_flat_index = TOTAL( (*bdf_ptr).polar_flat_set_elements[0:polar_flat_set-1] )
					endelse
					data_array[i].flat = FILE_BASENAME( (*bdf_ptr).polar_flat_path[polar_flat_index])
				endelse
			endif
			;
			data_array[i].status    = (*bdf_ptr).status[i]
			;
			color_array[*,i] = "white"
			;
			case data_array[i].type of
				"Skip": color = 'red'
				"**BAD**":	color = "red"
				"Bias":	color = "skyblue"
				"Dark": color = "grey68"
				"Pol-Img": color = "lawngreen"
				"Pol-Spec": color = "pink"
				"Spec": color = "plum"
				"Image": color = "yellow"
				"S-Argon": color = "slateblue"
				"Lin-On": color = "khaki light"
				"Lin-Off": color = "khaki dark"
				"I-Flat-On": color = "goldenrod light"
				"I-Flat-Off": color = "goldenrod dark"
				"S-Flat-On": color = "rosybrown light"
				"S-Flat-Off": color = "rosybrown dark"
				"P-Flat-On": color = "palegreen light"
				"P-Flat-Off": color = "palegreen dark"
				"PCal-On": color = "palegreen light"
				"PCal-Off": color = "palegreen dark"
				"R-Flat-Off": color = "steelblue dark"
				"R-Flat-On": color = "steelblue light"
				else: color = "white"
			endcase
			;
			;	also check status
			;
			if(data_array[i].status eq 'Skip') then color = 'goldenrod light'
			if(data_array[i].status eq 'Error') then color = 'red'
			;
			color_array[*,i] = color
	endfor
	;
	r_string = "Dark, Flat Correct Group List"
	option_msg = "       or put 'skip' in Status box to not process group(s)"
    new_struct = Dialog_Big_Table(Title="Group List for Dark, Flat Correct for Night "+$
    				STRING(format='(I8.8)',(*n_ptr).night_desig)+option_msg,Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
	;
	;	load new table data back into structure
	;
	nc = N_TAGS(new_struct)
	nr = N_ELEMENTS(new_struct)
	;
	;	note - permit change of status only
	;
	for ir = 0, nr-1 do begin
		if( new_struct[ir].status ne 'To Do') then (*bdf_ptr).status[ir] = 'Skip'
		;
		;	update Dark, Flat, BPM to allow user to change - must recover index for each, though
		;
		this_dark = new_struct[ir].dark
		ind_dark = WHERE( FILE_BASENAME((*bdf_ptr).dark_path) eq this_dark, count_dark)
		if(count_dark eq 1) then begin
			(*bdf_ptr).group_dark_index[ir] = ind_dark
		endif else begin
			(*bdf_ptr).group_dark_index[ir] = -1
		endelse
		;
		this_flat = new_struct[ir].flat
		ind_flat = WHERE( FILE_BASENAME((*bdf_ptr).flat_path) eq this_flat, count_flat)
		if(count_flat eq 1) then begin
			(*bdf_ptr).group_flat_index[ir] = ind_flat
		endif else begin
			(*bdf_ptr).group_flat_index[ir] = -1
		endelse
		;
		this_bpm = new_struct[ir].BPM
		ind_bpm = WHERE( FILE_BASENAME((*bdf_ptr).bpm_path) eq this_bpm, count_bpm)
		if(count_bpm eq 1) then begin
			(*bdf_ptr).group_bpm_index[ir] = ind_bpm
		endif else begin
			(*bdf_ptr).group_bpm_index[ir] = -1
		endelse
	endfor
	;
	if(obs_type eq 'phot') then P12_button_sensitize, Event, 2, 1
	if(obs_type eq 'pol') then begin
		;
		;	check for rollback images for this directory
		;
		;	1. test for the presence of a key subdirectory
		;
		test_dir = (*s_ptr).root_reduced + PATH_SEP() + "8_Linearity_Corrected_Images" + PATH_SEP()
		test_dir = test_dir + STRING((*n_ptr).night_desig,format='(I8)') + PATH_SEP() + 'Original_pol_images'
		;
		;print, "test_dir = ",test_dir
		;
		test_this = FILE_INFO(test_dir)

		;print, "test_this ", test_this
		;help, test_this, /struct
		;
		if(test_this.exists and test_this.directory) then begin
			;
			;	subdirectory is present - look for files
			;
			MSP_BDP_print_text, Event, "Found an Original Images sub-directory - testing for files"
			sub_list = FILE_SEARCH(test_dir+ PATH_SEP() + "*.*")
			n_sub = N_ELEMENTS(sub_list)
			if(n_sub eq 0) then begin
				MSP_BDP_print_text, Event, "Subdirectory was empty. Processing continues"
				goto, no_sub
			endif
			MSP_BDP_print_text, Event, "Subdirectory had "+STRING(n_sub)+" files."
			;
			;	ok, directory is present and images are present
			;
			;		turn on rollback base and its buttons, then exit to await further instructions
			;
			WIDGET_CONTROL, (*s_ptr).P12_base_wid, SENSITIVE=1
			P12_button_sensitize, Event, 6, 1
			P12_button_sensitize, Event, 7, 1
			P12_button_sensitize, Event, 5, 0
			;
			goto, P12_ST_end
		endif
		;
		no_sub:
		;
		P12_button_sensitize, Event, 5, 1
	endif
	;
	P12_ST_end:
end
;
;------------------------------------------------------------------
pro P12_Image_Rollback, Event, rollback_flag
	;
	;	if rollback_flag = 1, selects image rollback of the current directory
	;	if 0, continues processing
	;
	if(rollback_flag) then MSP_BDP_print_text, Event, "Image Rollback Requested - Please wait..."
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	bdf_ptr = PTR_struct.PTR_BDF
	;
	;	first, if no rollback, turn off the other button, turn on the processing
	;	button and exit
	;
	if(rollback_flag eq 0) then begin
		P12_button_sensitize, Event, 6, 0
		P12_button_sensitize, Event, 5, 1
		goto, PIR_end
	endif
	;
	;	rollback was requested.
	;
	;		turn off other button
	;
	P12_button_sensitize, Event, 7, 0
	;
	WIDGET_CONTROL, /HOURGLASS
	;
	;	Find directory again
	;
	LC_dir = (*s_ptr).root_reduced + PATH_SEP() + "8_Linearity_Corrected_Images" + PATH_SEP() + STRING((*n_ptr).night_desig,format='(I8)')
	Org_dir = LC_dir + PATH_SEP() + 'Original_pol_images'
	;
	;print, "test_dir = ",test_dir
	;
	test_this = FILE_INFO(Org_dir)
	;
	;print, "test_this ", test_this
	;help, test_this, /struct
	;
	if(test_this.exists and test_this.directory) then begin
		;
		;	subdirectory is present - look for files
		;
		;MSP_BDP_print_text, Event, "Found an Original Images sub-directory - testing for files"
		sub_list = FILE_SEARCH(Org_dir+ PATH_SEP() + "*.*")
		n_sub = N_ELEMENTS(sub_list)
		if(n_sub eq 0) then begin
			MSP_BDP_print_text, Event, "Subdirectory was empty. Processing continues"
			goto, no_sub1
		endif
		MSP_BDP_print_text, Event, "Subdirectory had "+STRING(n_sub)+" files."
		;
		WIDGET_CONTROL, /HOURGLASS
		;
		;	identify the file names of the files listed for correction on the View/Edit table
		;
		n_good_groups = (*bdf_ptr).n_groups
		group_index = (*bdf_ptr).group_index[0:n_good_groups-1]
		;
		;	look to the View/Edit list for the images to test for matches
		;
		lc_images = strarr(10000)
		n_images = 0L
		;
		copy_from_file = strarr(10000)
		copy_to_file = strarr(10000)
		n_copy = 0L
		;
		for ig = 0, n_good_groups-1 do begin
			ind_gp = group_index[ig]
			if((*bdf_ptr).status[ig] eq 'To Do') then begin
				this_n_images = (*n_ptr).groups[ind_gp].group_images
				n_images = n_images + this_n_images
				;
				for iilc = 0, this_n_images-1 do begin
					;
					;	rebuild the image names from the View/Edit list in the LC directory
					;
					lc_images[iilc] = LC_dir + PATH_SEP() + STRING((*n_ptr).night_desig,format='(I8)') + $
										'.'+ STRTRIM(STRING((*n_ptr).groups[ind_gp].group_list[iilc],format='(I4)'),2) + $
										'_LC.fits'
					;
					;	and whether any match to those on the Originals list
					;
					ind_m = WHERE(FILE_BASENAME(lc_images[iilc]) eq FILE_BASENAME(sub_list),nmatch)
					if(nmatch ne 0) then begin
						copy_from_file[n_copy] = sub_list[ind_m[0]]
						copy_to_file[n_copy] = lc_images[iilc]
						n_copy++
					endif
				endfor
			end
		endfor
		;
		;	any to copy?
		;
		if(n_copy eq 0) then begin
			MSP_BDP_print_text, Event, "No matching images in Originals directory. Processing Continues"
			P12_button_sensitize, Event, 6, 0
			P12_button_sensitize, Event, 5, 1
			goto, no_sub1
		endif
		;
		;	there were matching files,
		;
		;	start loop of forcibly copying from Orig to LC and deleting Orig
		;
		for ifile = 0, n_copy-1 do begin
			;			;
			FILE_COPY, copy_from_file[ifile], copy_to_file[ifile], /OVERWRITE
		endfor
		;
		MSP_BDP_print_text, Event, "Files copied Original->Lin. Corr. Directory"
		;
		WIDGET_CONTROL, /HOURGLASS
		;
		;	if we got here, all were written, so delete those files
		;
		FILE_DELETE, copy_from_file[0:n_copy-1]
		;
		;	now, test for the directory being empty
		;
		test_this = FILE_INFO(Org_dir)
		;
		if(test_this.exists and test_this.directory) then begin
			;
			any_sub_list = FILE_SEARCH(Org_dir+ PATH_SEP() + "*.*")
			any_sub = N_ELEMENTS(sub_list)
			if(any_sub eq 0) then begin
				MSP_BDP_print_text, Event, "Subdirectory empty. Deleting it"
				FILE_DELETE, Org_dir
			endif else begin
				MSP_BDP_print_text, Event, "Subdirectory not empty. Not deleting it"
			endelse
		endif
		;
		;	done, turn on processing button
		;
		P12_button_sensitize, Event, 5, 1
	endif
	;
	no_sub1:
	;
	PIR_end:
end
;
;-----------------------------------------------------------------
pro P12_Start_Image_Calculation, Event, obs_type
	;
	;	for every good group and every file, perform dark correction and flat-field
	;	corrections, using the BDP mask
	;
	MSP_BDP_print_text, Event, "Start Dark, Flat Corrections Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	f_ptr = PTR_struct.PTR_FITS
	bdf_ptr = PTR_struct.PTR_BDF
	rt_ptr = PTR_struct.PTR_RTime
	;
	n_groups = (*n_ptr).num_groups
	;
	;	that was all the groups for this night, now get the good groups
	;
	n_good_groups = (*bdf_ptr).n_groups
	group_index = (*bdf_ptr).group_index[0:n_good_groups-1]
	;
	night_desig = (*n_ptr).night_desig
	night_desig_text = STRTRIM(STRING(format='(i8.8)',night_desig),2)
	;
	dir_LC_night = (*s_ptr).root_reduced + '8_Linearity_Corrected_Images' + PATH_SEP() + $
					night_desig_text + PATH_SEP() + night_desig_text + '.'
	dir_out = (*s_ptr).root_reduced + '12_Science_Images' + PATH_SEP() + $
					night_desig_text + PATH_SEP()
	;
	if(obs_type eq 'pol') then dir_out = dir_out + 'polarimetry' + PATH_SEP()
	;
	test = FILE_INFO(dir_out)
	if(NOT(test.exists) or NOT(test.directory)) then begin
		MSP_BDP_print_text, Event, "Creating new output directory "+dir_out
		FILE_MKDIR, dir_out
	endif
	;
	;	clear zero hop reporting textboxes
	;
	WIDGET_CONTROL, (*s_ptr).P12_text_wid[0], SET_VALUE='0'
	WIDGET_CONTROL, (*s_ptr).P12_text_wid[1], SET_VALUE='  '
	n_zero_hop_groups = 0
	;
	;	compute the number of images to correct
	;
	n_images = 0L
	for ig = 0, n_good_groups-1 do begin
		ind = group_index[ig]
		if((*bdf_ptr).status[ig] eq 'To Do') then n_images = n_images + (*n_ptr).groups[ind].group_images
	endfor
	MSP_BDP_print_text, Event, "Starting Correction of "+STRING(format='(I)',n_images)+' images'
	;
	;	set up LED progress bar
	;
	n_LEDs = N_ELEMENTS( (*s_ptr).P12_LED_colors )
	LED_scale = float(n_images) / float(n_LEDs)
	if(LED_scale le 1.0) then LED_scale = 1.0
	(*s_ptr).P12_LED_colors[*] = 'yellow'
	if(n_images lt n_LEDs) then (*s_ptr).P12_LED_colors[n_images:n_LEDs-1] = 'black'
	if(obs_type eq 'phot') then WCT_LED_stack, (*s_ptr).P12_LED_wid[0], (*s_ptr).P12_LED_colors else $
			WCT_LED_stack, (*s_ptr).P12_LED_wid[1], (*s_ptr).P12_LED_colors
	;
	;	clear the indices of the bpm, dark, flat pointers
	;
	old_bpm_index = -1L
	old_dark_index = -1L
	old_flat_index = -1L
	old_polar_flat_index = -1L
	;
	;	for every good group
	;
	i_count_image = 0L
	(*bdf_ptr).n_fixed_groups = 0L
	for igg = 0, n_good_groups-1 do begin
		;
		this_LED = ROUND( float(i_count_image) / LED_scale)
		this_LED = 0 > ((n_LEDs-1) < this_LED)
		(*s_ptr).P12_LED_colors[this_LED] = 'orange'
		if(obs_type eq 'phot') then WCT_LED_stack, (*s_ptr).P12_LED_wid[0], (*s_ptr).P12_LED_colors else $
			WCT_LED_stack, (*s_ptr).P12_LED_wid[1], (*s_ptr).P12_LED_colors
		;
		;	check the status, skip if not "To Do"
		;
		this_status = (*bdf_ptr).status[igg]
		if(this_status ne 'To Do') then begin
			MSP_BDP_print_text, Event, "Skipping files in group Gr_"+STRTRIM(STRING(format='(i4.1)',group_index[igg]),2)
			(*s_ptr).P12_LED_colors[this_LED] = 'red'
			if(obs_type eq 'phot') then WCT_LED_stack, (*s_ptr).P12_LED_wid[0], (*s_ptr).P12_LED_colors else $
				WCT_LED_stack, (*s_ptr).P12_LED_wid[1], (*s_ptr).P12_LED_colors
			goto, igg_loop_end
		endif
		;
		;	for each of the BPM, Dark, Flat - compare the requested index for this group
		;	to the "old" index (telling us which image is currently open).
		;
		;	if there is a match, do not open a new file
		;	if no match, free the pointer and create a new variable, then read the image
		;
		this_bpm_index = (*bdf_ptr).group_bpm_index[igg]
		if(this_bpm_index ne old_bpm_index) then begin
			;
			;	free the existing pointer if the old_x_index was not -1L
			;
			if(old_bpm_index ne -1L) then begin
				PTR_FREE, (*bdf_ptr).bpm_image_ptr
				PTR_FREE, (*bdf_ptr).bpm_header_ptr
			endif
			;
			;	load old with new index, create variables
			;
			old_bpm_index = this_bpm_index
			bpm_path = (*bdf_ptr).bpm_path[this_bpm_index]
			header = HEADFITS(bpm_path, /silent)
			n_header_lines = N_ELEMENTS(header)
			(*s_ptr).raw_header[0:n_header_lines-1] = header
			(*s_ptr).raw_header[n_header_lines:(N_ELEMENTS((*s_ptr).raw_header))-1] = ''
			;
			MSP_BDP_Parse_FITS_Header, Event		; if needed to unpack stuff
			;
			nc = (*f_ptr).naxis1
			nr = (*f_ptr).naxis2
			;
			;	create bpm data, header pointers
			;
			(*bdf_ptr).bpm_image_ptr = PTR_NEW(intarr(nc,nr))
			(*bdf_ptr).bpm_header_ptr = PTR_NEW(strarr(n_header_lines))
			(*(*bdf_ptr).bpm_image_ptr) = readfits(bpm_path, (*(*bdf_ptr).bpm_header_ptr), /silent)
			;
			MSP_BDP_print_text, Event, "Loaded new bad pixel mask = "+FILE_BASENAME(bpm_path)
			;
		endif
		;
		;	same for dark
		;
		this_dark_index = (*bdf_ptr).group_dark_index[igg]
		if(this_dark_index ne old_dark_index) then begin
			;
			;	free the existing pointer if the old_x_index was not -1L
			;
			if(old_dark_index ne -1L) then begin
				PTR_FREE, (*bdf_ptr).dark_image_ptr
				PTR_FREE, (*bdf_ptr).dark_header_ptr
			endif
			;
			;	load old with new index, create variables
			;
			old_dark_index = this_dark_index
			dark_path = (*bdf_ptr).dark_path[this_dark_index]
			header = HEADFITS(dark_path, /silent)
			n_header_lines = N_ELEMENTS(header)
			(*s_ptr).raw_header[0:n_header_lines-1] = header
			(*s_ptr).raw_header[n_header_lines:(N_ELEMENTS((*s_ptr).raw_header))-1] = ''
			;
			MSP_BDP_Parse_FITS_Header, Event		; if needed to unpack stuff
			;
			nc = (*f_ptr).naxis1
			nr = (*f_ptr).naxis2
			;
			;	create dark data, header pointers
			;
			(*bdf_ptr).dark_image_ptr = PTR_NEW(fltarr(nc,nr))
			(*bdf_ptr).dark_header_ptr = PTR_NEW(strarr(n_header_lines))
			(*(*bdf_ptr).dark_image_ptr) = readfits(dark_path, (*(*bdf_ptr).dark_header_ptr), /silent)
			;
			MSP_BDP_print_text, Event, "Loaded new dark image = "+FILE_BASENAME(dark_path)
			;
		endif
		;
		;	same for flat
		;
		if(obs_type eq 'phot') then begin
			this_flat_index = (*bdf_ptr).group_flat_index[igg]
			if(this_flat_index ne old_flat_index) then begin
				;
				;	free the existing pointer if the old_x_index was not -1L
				;
				if(old_flat_index ne -1L) then begin
					PTR_FREE, (*bdf_ptr).flat_image_ptr
					PTR_FREE, (*bdf_ptr).flat_header_ptr
				endif
				;
				;	load old with new index, create variables
				;
				old_flat_index = this_flat_index
				flat_path = (*bdf_ptr).flat_path[this_flat_index]
				header = HEADFITS(flat_path, /silent)
				n_header_lines = N_ELEMENTS(header)
				(*s_ptr).raw_header[0:n_header_lines-1] = header
				(*s_ptr).raw_header[n_header_lines:(N_ELEMENTS((*s_ptr).raw_header))-1] = ''
				;
				MSP_BDP_Parse_FITS_Header, Event		; if needed to unpack stuff
				;
				nc = (*f_ptr).naxis1
				nr = (*f_ptr).naxis2
				;
				;	create dark data, header pointers
				;
				(*bdf_ptr).flat_image_ptr = PTR_NEW(fltarr(nc,nr))
				(*bdf_ptr).flat_header_ptr = PTR_NEW(strarr(n_header_lines))
				(*(*bdf_ptr).flat_image_ptr) = readfits(flat_path, (*(*bdf_ptr).flat_header_ptr), /silent)
				;
				MSP_BDP_print_text, Event, "Loaded new flat field image = "+FILE_BASENAME(flat_path)
				;
			endif
		endif
		;
		;	load the correct polar flat set of images
		;--------------**
		if(obs_type eq 'pol') then begin
			this_polar_flat_index = (*bdf_ptr).group_polar_flat_index[igg]
			if(this_polar_flat_index ne old_polar_flat_index) then begin
				;
				;	free the existing pointer if the old_x_index was not -1L
				;
				if(old_polar_flat_index ne -1L) then begin
					PTR_FREE, (*bdf_ptr).polar_flat_image_ptr
					PTR_FREE, (*bdf_ptr).polar_flat_header_ptr
				endif
				;
				;	load old with new index, create variables
				;
				old_polar_flat_index = this_polar_flat_index
				;
				;	need to read the full polarimetry set
				;
				;		start with HWP=0 image
				;
				if(this_polar_flat_index eq 0) then begin
					first_image_index = 0
					n_in_set = (*bdf_ptr).polar_flat_set_elements[0]
				endif else begin
					first_image_index = TOTAL( (*bdf_ptr).polar_flat_set_elements[0:this_polar_flat_index-1])
					n_in_set = (*bdf_ptr).polar_flat_set_elements[this_polar_flat_index]
				endelse
				;
				;	get mean_PA
				;
				polar_flat_mean_PA = (*bdf_ptr).polar_flat_mean_PA[this_polar_flat_index]
				;
				polar_flat_path_0 = (*bdf_ptr).polar_flat_path[first_image_index]
				header = HEADFITS(polar_flat_path_0, /silent)
				n_header_lines = N_ELEMENTS(header)
				(*s_ptr).raw_header[0:n_header_lines-1] = header
				(*s_ptr).raw_header[n_header_lines:(N_ELEMENTS((*s_ptr).raw_header))-1] = ''
				;
				MSP_BDP_Parse_FITS_Header, Event		; if needed to unpack stuff
				;
				nc = (*f_ptr).naxis1
				nr = (*f_ptr).naxis2
				;
				;	create dark data, header pointers
				;
				(*bdf_ptr).polar_flat_image_ptr = PTR_NEW(REPLICATE({image: fltarr(nc,nr)},n_in_set))
				(*bdf_ptr).polar_flat_header_ptr = PTR_NEW(REPLICATE({header: strarr(300)},n_in_set))
				;
				;	read images and headers
				;
				polar_flat_HWP = fltarr(n_in_set)
				polar_path = strarr(n_in_set)
				for i_in_set = 0, n_in_set-1 do begin
					polar_path[i_in_set] = (*bdf_ptr).polar_flat_path[first_image_index+i_in_set]
					(*(*bdf_ptr).polar_flat_image_ptr)[i_in_set].image = readfits(polar_path[i_in_set], header, /silent)
					n_header_lines = N_ELEMENTS(header)
					(*(*bdf_ptr).polar_flat_header_ptr)[i_in_set].header[0:n_header_lines-1] = header
					;
					;	load the HWP angles
					;
					polar_flat_HWP[i_in_set] = normal_angle(FXPAR(header, "HWPANGLE"))
				endfor
				;
				MSP_BDP_print_text, Event, "Loaded "+STRING(format='(I)',n_in_set)+' Polar Flat Fields from set '+$
									STRING(format='(I)',this_polar_flat_index)
				;
			endif
		endif
		;
		;
		(*s_ptr).P12_LED_colors[this_LED] = 'blue'
		if(obs_type eq 'phot') then WCT_LED_stack, (*s_ptr).P12_LED_wid[0], (*s_ptr).P12_LED_colors else $
			WCT_LED_stack, (*s_ptr).P12_LED_wid[1], (*s_ptr).P12_LED_colors
		;
		;	for every file in the group
		;
		n_images = (*n_ptr).groups[group_index[igg]].group_images
		image_list = (*n_ptr).groups[group_index[igg]].group_list[0:n_images-1]
		this_group_number = (*n_ptr).groups[group_index[igg]].group_id
		this_group_name = (*n_ptr).groups[group_index[igg]].group_object
		;
		this_group_filter = (*n_ptr).groups[group_index[igg]].group_filter
		;
		;-------------------------------------------------------------------------------------------------------
		;
		;	if this is a polarimetry group - then perform the Zero Hop test on it, fixing any images
		;		that are flagged as needing fixing. The unfixed versions will be written to a subdirectory,
		;		and the fixed versions will be written "in place"  - DPC 20091223
		;
		;	only applies to H-pol data - no other HWP has double home positions (DPC 20111103)
		;
		;	added a background level test - of too few background counts, this test will fail, so
		;	skip it. NB, this could be a problem for pol stds images... (DPC 20120206)
		;
		test_H_pol = this_group_filter eq 'H-Pol'
		test_K_pol = this_group_filter eq 'K-Pol'
		sky_test = 50.0									; ADUs average per image - a guess
		;
		if(test_H_pol eq 1 and test_K_pol eq 0) then pa_offset = 67.35
		if(test_H_pol eq 0 and test_K_pol eq 1) then pa_offset = 135.0 ; ????
		;
		if( (*s_ptr).P12_Ignore_ZHT_flag eq 1 or ~test_H_pol) then begin
			MSP_BDP_print_text, Event, "Skipping Zero Hop test for pol group #"+STRING(this_group_number,format='(I)')+' = '+this_group_name
		endif
		;
		if(obs_type eq 'pol' and (*s_ptr).P12_Ignore_ZHT_flag eq 0 and test_H_pol) then begin
			;
			;	build full pathnames to LC images
			;
			image_no_text = STRTRIM(STRING(format='(i5.3)',image_list),2)
			input_pathnames = dir_LC_night + image_no_text + '_LC.fits'
			;
			;	read all images in group to get the mean background level
			;
			get_group_sky, Event, input_pathnames, group_sky
			;
			MSP_BDP_print_text, Event, "Group "+STRING(this_group_number,format='(I)')+' = '+this_group_name+" has mean sky = "+STRING(group_sky,format='(F7.2)')+" counts"
			;
			if(group_sky ge sky_test) then begin
				;
				MSP_BDP_print_text, Event, "Performing Zero Hop test for pol group #"+STRING(this_group_number,format='(I)')+' = '+this_group_name

				;
				this_debug = 0
				;
				check_dither = 1
				;
				;	turn on hourglass
				;
				WIDGET_CONTROL, /HOURGLASS
				hwp_fidelity_test, Event, input_pathnames, check_dither, fidelity_out, fold_angle, mean_PA, $
							pa_dith, s_pa_dith, dither_vector, this_debug, DITHER=dither_class
				;
				;	special case, if return with mean_PA = -99.9 - too few images to do the test, dump the entire group
				;
				if(mean_PA eq -99.9) then begin
					n_in_group = N_ELEMENTS(input_pathnames)
					MSP_BDP_print_text, Event, 'Warning - too few HWP angles in this group '+STRING(n_in_group,format='(I)')+' to Zero Hop test - dumping group'
					goto, igg_loop_end
				endif
				;
				;	compute the mean PA also from the individual PAs, if present
				;
				if(N_ELEMENTS(pa_dith) eq 1) then begin
					if(pa_dith eq -99) then begin
						MSP_BDP_print_text, Event, 'Warning - too few HWP angles in this group '+STRING(n_in_group,format='(I)')+' to Zero Hop test - dumping group'
						goto, igg_loop_end
					endif
				endif
				;
				n_dith = N_ELEMENTS(pa_dith)
				;
				if(n_dith gt 1) then begin
					pa_dith_moment = MOMENT(pa_dith)
					pa_dith_mean = pa_dith_moment[0]
					pa_dith_rms = sqrt(pa_dith_moment[1])
					dither_vector -= 1							; restart from zero
				endif else begin
					pa_dith_mean = REFORM(pa_dith[*])
					pa_dith_rms = 10.0		; force testing
					dither_vector -= 1
				endelse
				;
				rms_criterion = 7.0
				;
				if(pa_dith_rms gt rms_criterion) then begin
					;
					;
					MSP_BDP_print_text, Event, 'Zero Hop testing found dither sets needing fixing in this group'
					;
					;	update counter textbox and group list textbox
					;
					n_zero_hop_groups++
					WIDGET_CONTROL, (*s_ptr).P12_text_wid[0],SET_VALUE=STRING(n_zero_hop_groups)
					;
					WIDGET_CONTROL, (*s_ptr).P12_text_wid[1],/APPEND,SET_VALUE='G#'+STRING(this_group_number,format='(I3.3)')+$
							' - '+this_group_name+' had PA by dither RMS = '+STRING(pa_dith_rms)
					;
					;	print the individual PAs and the number of 45 deg offsets or rejections
					;
					dith_45_offsets = fltarr(n_dith)
					dith_45_use = intarr(n_dith)
					;
					for idith = 0, N_ELEMENTS(pa_dith)-1 do begin
						;
						;	mod 45?
						;
						this_pa = pa_dith[idith]
						diff_pa = pa_offset - this_pa
						;
						resid = (diff_pa + 720.0) MOD 45
						if(resid gt 25.0) then resid = resid - 45.0
						;
						;	test for being within either 10 deg or 2 sigma, but always less than 22.5 deg
						;
						resid_test = 10.0 > (2.0*s_pa_dith[idith] < 22.5)
						if(abs(resid) le resid_test) then begin
							dith_45_use[idith] = 1
						endif else begin
							dith_45_use[idith] = 0
						endelse
						;
						;	and the number of 45 degree increments to add to each HWP angle
						;
						n_45 = ROUND(diff_pa / 45.0)
						if(n_45 lt 0) then n_45 += 4
						dith_45_offsets[idith] = n_45
						;
						MSP_BDP_print_text, Event, "  For dither = "+STRING(idith,format='(I1)')+$
								" PA, sPA, dPA = "+STRING(pa_dith[idith],s_pa_dith[idith],resid,$
								format='(F6.1,"+/-",F5.1,2x,F6.1)')+"  N_45, Use = "+STRING(n_45,dith_45_use[idith],$
								format='(I4,1x,I1)')
					endfor
					;
					;	need to do correction by dither
					;
				endif
				;
				MSP_BDP_print_text, Event,"  found mean_PA = "+STRING(mean_PA)
				;
				;	if the individual PAs are different and the ZHT-by-dither flag is set, must correct
				;	images on a dither by dither basis - else, do entire set
				;
				if( (*s_ptr).P12_zht_dither_flag eq 1 and pa_dith_rms gt rms_criterion) then begin
					;
					;	do the dither correction
					;
					;	first, copy original LC images into subdirectory for safe keeping
					;
					P12_save_images_to_subdirectory, Event, input_pathnames
					;
					for idith = 0, n_dith-1 do begin
						;
						;	send each set of 16 images for this dither off for repairs
						;
						these_inds = WHERE(dither_vector eq idith, n_in_set)
						if(n_in_set gt 0) then begin
							;
							;	3 options: (1) leave set alone; (2) correct set; (3) mark set for deletion
							;
							if(dith_45_use[idith] eq 0) then begin
								;
								;	mark this set for deletion - add a 'DELE_ME' keyword to the LC image
								;
								for i_this = 0, n_in_set-1 do begin
									;
									P12_mark_for_deletion, Event, input_pathnames[these_inds[i_this]]
									;
								endfor
								;
								MSP_BDP_print_text, Event, 'For Dither '+STRING(idith,format='(I1)')+$
									' marked all images for deletion'
							endif else begin
								if(dith_45_offsets[idith] gt 0) then begin
									;
									;	correct this set
									;
									for i_this = 0, n_in_set-1 do begin
										;
										n_45 = dith_45_offsets[idith]
										;
										image = input_pathnames[these_inds[i_this]]
										P12_fix_header_HWP_angle, Event, image, n_45
										;
									endfor
									;
									MSP_BDP_print_text, Event, 'For Dither '+STRING(idith,format='(I1)')+$
											' added '+STRING(n_45*45,format='(I3)')+' degrees of HWP angle in correction'
									;
								endif else begin
									;
									;	for non-correction, do nothing
									;
									MSP_BDP_print_text, Event, 'For Dither '+STRING(idith,format='(I1)')+$
											' no HWP correction was needed'
								endelse
							endelse
							;
							;	next dither set
							;
						endif
					endfor
				endif else begin
					;
					;	the full image set correction
					;
					;
					;	first, locate any really odd angles in dithers (fidelity = 2) - these ought to be
					;	deleted, if the signal is really strong
					;
					ind_odd = WHERE(fidelity_out eq 2, nodd)
					if(nodd gt 0) then begin
						odd_dith = WHERE(dither_class eq 2, nodd_d)
						if(nodd_d gt 0) then begin
							odd_str = STRING(odd_dith+1,format='(6(I1,","))')
						endif else begin
							odd_str = ''
						endelse
						;
						WIDGET_CONTROL, (*s_ptr).P12_text_wid[1],/APPEND,SET_VALUE='G#'+STRING(this_group_number,format='(I3.3)')+$
						        	' - '+this_group_name+' had odd angles in dithers #'+odd_str+' deletion recommended'
					endif
					;
					;	if any are marked for change (fidelity = 1), then set up save directory, copy files, and
					;	change the headers of the ones left behind
					;
					ind_fid = WHERE(fidelity_out eq 1, nfix)
					;
					bad_dith = WHERE(dither_class eq 1,nd)
					if(nd gt 0) then begin
						bad_str = STRING(bad_dith+1,format='(6(I1,","))')
					endif else begin
						bad_str = ''
					endelse
					;
					if(nfix gt 0) then begin
						;
						MSP_BDP_print_text, Event, 'Zero Hop testing found '+STRING(nfix)+' images needing fixing in this group'
						;
						;	update counter textbox and group list textbox
						;
						n_zero_hop_groups++
						WIDGET_CONTROL, (*s_ptr).P12_text_wid[0],SET_VALUE=STRING(n_zero_hop_groups)
						;
						WIDGET_CONTROL, (*s_ptr).P12_text_wid[1],/APPEND,SET_VALUE='G#'+STRING(this_group_number,format='(I3.3)')+$
								' - '+this_group_name+' had '+STRING(nfix,format='(I3)')+' fixes in dithers #'+bad_str
						;
						P12_save_images_to_subdirectory, Event, input_pathnames[ind_fid]
						;
						; and fix the files
						;
						for i_fid = 0, nfix-1 do begin
							;
							;	read the current one
							;
							n_45 = 1
							;
							;	and increment by one 45 degree step of HWP phase (not angle)
							;
							P12_fix_header_HWP_angle, Event, input_pathnames[ind_fid[i_fid]], n_45
							;
						endfor
						;
						MSP_BDP_print_text, Event, 'Rewrote FITS headers for '+STRING(nfix)+' polarimetry images'
						;
					endif
				endelse
			endif else begin
				MSP_BDP_print_text, Event, "Skipping Zero Hop test for pol group #"+STRING(this_group_number,format='(I)')+' = '+this_group_name
			endelse
		endif
	;
	;-------------------------------------------------------------------------

		for ii = 0, n_images-1 do begin
			this_LED = ROUND( float(i_count_image) / LED_scale)
			this_LED = 0 > ((n_LEDs-1) < this_LED)
			(*s_ptr).P12_LED_colors[this_LED] = 'white'
			if(obs_type eq 'phot') then WCT_LED_stack, (*s_ptr).P12_LED_wid[0], (*s_ptr).P12_LED_colors else $
				WCT_LED_stack, (*s_ptr).P12_LED_wid[1], (*s_ptr).P12_LED_colors
			;
			;	create input filename for linearity corrected input file
			;
			image_no_text = STRTRIM(STRING(format='(i5.3)',image_list[ii]),2)
			input_pathname = dir_LC_night + image_no_text + '_LC.fits'
			;
			;	prepare output pathname and directory
			;
			output_pathname = dir_out + night_desig_text + '.' + image_no_text + '_LDFC.fits'
			;
			;	read input image
			;
			input_image = readfits(input_pathname, input_header, /silent)
			;
			;	test for presence of 'DELE_ME' keyword - if present, skip this image
			;
			val = SXPAR(input_header, 'DELE_ME', COUNT=n_del)
			if(n_del gt 0) then begin
				MSP_BDP_print_text, Event, 'Removing image '+FILE_BASENAME(input_pathname)+' from Step 12'
				;
				(*s_ptr).P12_LED_colors[this_LED] = 'orange'
				goto, image_del_end
			endif
			;
			;	if polar, get HWP angle and match to list
			;
			if(obs_type eq 'pol') then begin
				n_header_lines = N_ELEMENTS(input_header)
				(*s_ptr).raw_header[0:n_header_lines-1] = input_header
				MSP_BDP_Parse_FITS_Header, Event
				MSP_BDP_Parse_HWP_Angle, Event, HWP_angle
				;
				dif = ABS(HWP_angle - polar_flat_HWP[*])
				min_dif = MIN(dif, i_hwp)
				;
				if(N_ELEMENTS(i_hwp) eq 1) then begin
					i_hwp = i_hwp[0]
				endif else begin
					MSP_BDP_print_text, Event, "Error - failed to match image"+ $
										" HWP angle to polar flat list. Skipping image " +  $
										FILE_BASENAME(input_pathname)
					(*s_ptr).P12_LED_colors[this_LED] = 'pink'
					goto, image_del_end
				endelse
				;
				;	i_hwp is the index to the polar flat image
				;
			endif
			;
			;	determine whether this was a coadded image
			;
			coadds = SXPAR(input_header,'NUMCOADD',count=n_f_coadds)
			if(n_f_coadds ne 1) then begin
			   if(n_f_coadds eq 0) then begin
			       coadds = 1
			       FXADDPAR, input_header, 'NUMCOADD', 0, 'Number of actual frames co-added together'
			   endif else stop
			endif
			;
			if(coadds le 1) then coadds = 1
			;
			;	first math operation, subtract dark - if coadds =/ 1 scale dark first
			;
			if(coadds eq 1) then begin
				dif_image = safe_math(input_image, (*(*bdf_ptr).dark_image_ptr),$
						(*(*bdf_ptr).bpm_image_ptr), '-' )
			endif else begin
				scaled_dark = safe_math( (*(*bdf_ptr).dark_image_ptr), coadds, (*(*bdf_ptr).bpm_image_ptr), '*')
				dif_image = safe_math(input_image, scaled_dark,$
						(*(*bdf_ptr).bpm_image_ptr), '-' )
			endelse
			;
			;	then division by flat
			;
			if(obs_type eq 'phot') then begin
				output_image = safe_math(dif_image, (*(*bdf_ptr).flat_image_ptr), $
						(*(*bdf_ptr).bpm_image_ptr), '/' )
			endif else begin
				output_image = safe_math(dif_image, (*(*bdf_ptr).polar_flat_image_ptr)[i_hwp].image, $
						(*(*bdf_ptr).bpm_image_ptr), '/' )
			endelse
			;
			;	update header and write to disk
			;
			;	rewrite object name in case it was edited at the group table level
			;
			FXADDPAR, input_header, "OBJECT", (*n_ptr).groups[this_group_number].group_object
			FXADDPAR, input_header, "FILENAME", FILE_BASENAME(output_pathname), "MSP_BDP Part 12 Generated"
			FXADDPAR, input_header, "DIRNAME", FILE_DIRNAME(output_pathname), "MSP_BDP Storage Location"
			FXADDPAR, input_header, "HISTORY", "Science Data Generated with MSP-BDP on "+SYSTIME()
			FXADDPAR, input_header, "HISTORY", "  reduced by "+(*s_ptr).who
			FXADDPAR, input_header, "HISTORY", "  Using Bad Pixel Mask "+FILE_BASENAME(bpm_path)
			FXADDPAR, input_header, "HISTORY", "  and Dark Corrected with image "+FILE_BASENAME(dark_path)
			if(coadds ne 1) then begin
				FXADDPAR, input_header, "HISTORY", "  with scaling by coadds = "+STRING(coadds, format='(I3)')
			endif
			if(obs_type eq 'phot') then begin
				FXADDPAR, input_header, "HISTORY", "  and Flat-Field Corrected with image "+FILE_BASENAME(flat_path)
			endif else begin
				FXADDPAR, input_header, "HISTORY", "  and Polar FF Corrected with "+FILE_BASENAME(polar_path[i_hwp])
			endelse
			;
			;	add MSP-BDP classification findings to the FITS header
			;
			FXADDPAR, input_header, "MB_NIGHT", night_desig, "MSP-BDP Night Designation"
			FXADDPAR, input_header, "MB_GROUP", this_group_number, "MSP-BDP Group Number in Night"
			;
			;	for the rest of the classification findings, must look to the night summary for this
			;	image
			;
			n_night_images = (*n_ptr).num_images
			ind = WHERE((*n_ptr).image_list[0:n_night_images-1] eq image_list[ii],count)
			if(count ne 1) then begin
				MSP_BDP_print_text, Event, "ERROR - no image match for header data for image "+STRING(format='(I)',image_list[ii])
			endif else begin
				k = ind[0]
				FXADDPAR, input_header, "MB_ITYPE", (*n_ptr).image_type[k], "MSP-BDP Image Type"
				FXADDPAR, input_header, "MB_FILTR", (*n_ptr).image_filter[k],"MSP-BDP Image Filter Type"
				FXADDPAR, input_header, "MB_FRAME", (*n_ptr).image_frame[k],"MSP-BDP Subframe or Full mode"
				FXADDPAR, input_header, "MB_HWP",   normal_angle((*n_ptr).image_HWP_angle[k]),"MSP-BDP HWP Angle in degrees"
				FXADDPAR, input_header, "MB_SKJMP", (*n_ptr).image_ski_jump[k],"MSP-BDP Ski Jump Type"
				FXADDPAR, input_header, "MB_CAMRA", (*n_ptr).image_camera[k],"MSP-BDP Camera Type"
				FXADDPAR, input_header, "MB_RDMOD", (*n_ptr).image_readout_mode[k],"MSP-BDP Readout Mode"
				FXADDPAR, input_header, "MB_RDTIM", (*rt_ptr).read_time_value,"MSP-BDP Readout Time Value"
				FXADDPAR, input_header, "MB_WHO",   (*s_ptr).who, "MSP-BDP Data Reduction Person"
			endelse
			;
			;	if output_image is double precision, convert to single before writing
			;
			this_type = SIZE(output_image,/TYPE)
			if(this_type eq 5) then output_image = FLOAT(output_image)
			;
			;	test for NaNs..
			;
			ind_nan = WHERE(~FINITE(output_image), n_nan)
			if(n_nan ne 0) then begin
				output_image[ind_nan] = -1.0e6
			endif
			;
			writefits, output_pathname, output_image, input_header
			;
			MSP_BDP_print_text, Event, "Wrote image "+FILE_BASENAME(output_pathname)+" to disk"
			;
			(*s_ptr).P12_LED_colors[this_LED] = 'green'
			;
			image_del_end:
			;
			if(obs_type eq 'phot') then WCT_LED_stack, (*s_ptr).P12_LED_wid[0], (*s_ptr).P12_LED_colors else $
				WCT_LED_stack, (*s_ptr).P12_LED_wid[1], (*s_ptr).P12_LED_colors
			i_count_image++
			;
			image_loop_end:
		endfor
		;
;		(*s_ptr).P12_LED_colors[this_LED] = 'green'
;		WCT_LED_stack, (*s_ptr).P12_LED_wid[0], (*s_ptr).P12_LED_colors
		(*bdf_ptr).n_fixed_groups++
		;
		igg_loop_end:
		;
	endfor
	;
	MSP_BDP_print_text, Event, "Science Corrections Done for Night "+night_desig_text
	;
	P12_Save_Night_Summary, Event
end
;
;--------------------------------------------------------------------------------------
pro P12_Save_Night_Summary, Event
	;
	;	saves and updated night summary, based on properties
	;	of the number of shifts recommended or completed
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	bdf_ptr = PTR_struct.PTR_BDF
	;
	;	how may good groups were processed?
	;
	n_fix = (*bdf_ptr).n_fixed_groups
	;
	this_night = (*n_ptr).night_desig
	;
	;	which one in the summary?
	;
	num_nights = (*s_ptr).num_nights
	ind = WHERE(this_night eq (*s_ptr).night_date[0:num_nights-1], count)
	if(count ne 0) then night_index = ind[0] else begin
		MSP_BDP_print_text, Event, "Error - No matching night in P12..."
		goto, P12SNS_end
	endelse
	;
	(*s_ptr).night_who[night_index] = (*s_ptr).who
	(*s_ptr).night_step[night_index] = 12
	(*s_ptr).night_comment[night_index] = STRTRIM(STRING(format='(i4.1)',n_fix),2)+" Groups Finished"
	;
	P1_Write_Summary_File, Event
	;
	P12SNS_end:
end
