;
;	Part 3 Event Handler Routines - Kokopeli Crack Analysis
;
;		20121031 - added LL quad test  (Jordan's code)
;		20121103 - too many false positives for spec, turning off
;
;-----------------------------------------------------------------
pro P3_Select_Night, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	;
	wid = (*s_ptr).P3_droplist_night_wid
	;
	night_index = Event.index
	;
	;	index in the droplist is not enough, as we might not have done all the part 2
	;	stuff.... So, need to get the night designation from the list
	;
	WIDGET_CONTROL, wid, GET_VALUE = vector_night_desig
	selected_night_desig = vector_night_desig[night_index]
	;
	;print, "in droplist, index = ",night_index
	;
	;night_desig = (*s_ptr).night_date[night_index]
	night_desig = selected_night_desig
	night_desig_text = STRING(format='(I8.8)',night_desig)
	;
	;	reset buttons and LEDs
	;
	for iw = 0, N_ELEMENTS( (*s_ptr).P3_button_wid)-2 do begin	; let checkbox remain sensitive
		if(iw le 0) then sense = 1 else sense = 0
		P3_button_sensitize, Event, iw, sense
	endfor
	;
	for iw = 0, N_ELEMENTS( (*s_ptr).P3_LED_wid)-1 do begin
		if(iw le 0) then sense = 1 else sense = 0
		P3_LED_sensitize, Event, iw, sense
	endfor
	;
	;	read in that night's save file
	;
	MSP_BDP_print_text, Event, "Opening Night Summary for night = "+night_desig_text
	;
	MSP_BDP_Create_Night_File_Path, Event, night_desig, night_file_path
	MSP_BDP_Restore_Night_File, Event, night_file_path, description, ierr
	;
	if(ierr ne 0) then begin
		MSP_BDP_print_text, Event, "Could not restore night "+night_desig_text+" - skipping"
		goto, P3SN_end
	endif
	;
	;	must redo pointers
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	raw_root = (*s_ptr).root_raw
	;
	;	capture number of groups
	;
	ngroups = (*n_ptr).num_groups
	ngroups_text = STRING(format='(I)',ngroups)
	MSP_BDP_print_text, Event, "Restored "+night_desig_text+" with "+ngroups_text+" groups"
	;
	;	turn on first button and progress bar
	;
	P3_button_sensitize, Event, 0, 1
	P3_LED_sensitize, Event, 0, 1
	;
	P3SN_end:
end
;
;----------------------------------------------------
pro P3_Start_Kokopeli, Event
	;
	;	build assessment of the number of images that need to be examined
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	c_ptr = PTR_struct.PTR_Column
	;
	raw_root = (*s_ptr).root_raw
	reduced_root = (*s_ptr).root_reduced
	;
	this_night = (*n_ptr).night_desig
	this_night_text = STRING(format='(I8.8)',this_night)
	;
;	;	if a repaired image directory for this night is present, delete it to prepare for a new one
;	;
;	koko_night_dir = reduced_root + '3_Kokopeli_Repaired_Images' + PATH_SEP() + this_night_text
;	test = FILE_INFO(koko_night_dir)
;	if( test.exists ) then begin
;		;
;		print, "ready to issue FILE_DELETE for directory = ",koko_night_dir
;		;
;		;	if this works correctly, do the full command:
;		;
;		;	FILE_DELETE, koko_night_dir, /QUIET, /RECURSIVE
;	endif
	;
	;	count images for rough progress estimate
	;
	total_raw_images = (*n_ptr).num_images
	;
	;	clear LED progress stack
	;
	(*s_ptr).P3_LED_colors[*] = "yellow"
	n_LED = N_ELEMENTS( (*s_ptr).P3_LED_colors )
	LED_scale = float(total_raw_images) / float(n_LED)
	if(LED_scale lt 1.0) then LED_scale = 1.0
	if(total_raw_images lt n_LED) then (*s_ptr).P3_LED_colors[total_raw_images:n_LED-1] = 'black'
	WCT_LED_stack, (*s_ptr).P3_LED_wid[0], (*s_ptr).P3_LED_colors
	;
	;----------
	;
	image_counter = 0L
	last_LED = 0
	;
	need_shift_counter = 0L
	;
	;	put the night's designator in the night window
	;
	night_desig = this_night
	night_text = this_night_text
	;
	;	capture number of groups
	;
	ngroups = (*n_ptr).num_groups
	;
	;	turn on hourglass
	;
	;WIDGET_CONTROL, /HOURGLASS
	;
	;	for each group
	;
	n_bad_col = 0L
	;
	for igroup = 0, ngroups-1 do begin
		;
		;	load the group ID into the group textbox
		;
		group_id = (*n_ptr).groups[igroup].group_id
		group_id_text = STRING(format='(I3.1)',group_id)
		WIDGET_CONTROL, (*s_ptr).P3_window_wid[0], SET_VALUE=group_id_text
		;
		;	if the group type is **BAD**, skip it
		;
		group_type = (*n_ptr).groups[igroup].group_type
		if( group_type eq '**BAD**' or group_type eq 'Skip') then begin
			MSP_BDP_print_text, Event, "Group "+STRING(format='(I)',igroup)+" is BAD, so skipping it"
			goto, group_end
		endif
		;
		;	load the image number table
		;
		this_group_size = (*n_ptr).groups[igroup].group_images
		this_group_images = (*n_ptr).groups[igroup].group_list[0:this_group_size-1]
		;
		;	for every image in this group
		;
		for jimage = 0, this_group_size-1 do begin
			;
			;	load the image number into the text box
			;
			image_number = this_group_images[jimage]
			image_number_text = STRTRIM(STRING(format='(I5.3)',image_number),2)
			WIDGET_CONTROL, (*s_ptr).P3_window_wid[1], SET_VALUE=image_number_text
			;
			;	bump consecutive counter and test whether to change LED progress stack
			;
			this_LED = ROUND(float(image_counter) / LED_scale)
			this_LED = 0 > ((n_LED-1) < this_LED)
			(*s_ptr).P3_LED_colors[this_LED] = "orange"
			WCT_LED_stack, (*s_ptr).P3_LED_wid[0], (*s_ptr).P3_LED_colors
			image_counter++
			;
			;	pathname to CDS directory
			;
			;	1. rescue cds/raw1 index pointer for this image
			;
			ind = WHERE( image_number eq (*n_ptr).image_list[0:total_raw_images-1], count)
			if(count ne 1) then begin
				MSP_BDP_print_text, Event, "No match of group image number to night list for "+image_number_text+". skipping"
				goto, image_loop_end
			endif
			;
			;	if this image is of type "**BAD**" or "skip" then skip it
			;
			image_type = (*n_ptr).image_type[ind[0]]
			if(image_type eq "**BAD**" or STRUPCASE(image_type) eq "SKIP") then begin
				MSP_BDP_print_text, Event, "Image number "+STRING(format='(I)',image_number)+" is BAD or skip-designated, so skipping"
				goto, image_loop_end
			endif
			;
			;		now have the image number index, use it to get the cds/raw1 path index
			;
			;	20100126 - reset image root index to raw data directory
			;
			(*n_ptr).image_root_index[ind[0]] = 0L
			;
			;
			cds_raw1_index = (*n_ptr).image_root_index[ind[0]]
			;
			if(cds_raw1_index eq 0) then begin
				CDS_root = (*s_ptr).root_raw
				RAW1_root = (*s_ptr).root_raw
			endif else begin
				CDS_root = (*s_ptr).root_reduced
				RAW1_root = (*s_ptr).root_reduced
			endelse
			;
			CDS_dir = CDS_root + (*n_ptr).paths_cds_root[cds_raw1_index]
			RAW1_dir = RAW1_root + (*n_ptr).paths_cds_root[cds_raw1_index]
			;
			;		A. match image number to night's list of image numbers
			;		B. look up this image number's CDS and RAW1 directory paths
			;		C. build image pathnames, as needed
			;
			;	form pathname to image
			;
			image_pathname = fix_path( FILEPATH(night_text+"."+image_number_text+".fits", ROOT_DIR = CDS_dir) )
			;
			;	and put on heap for LL quad test - 20121031
;			;
;			ll_ptr = PTR_NEW(readfits(image_pathname))
;			ll_status = test_bad_lower_left_quadrant(ll_ptr)		; 0 = good; 1 = first bad; 2 = other bad
;			;
;			if(ll_status ne 0) then begin
;				if(ll_status eq 1) then begin
;					MSP_BDP_print_text, Event, 'Found First Image with Bad LL Quad = '+FILE_BASENAME(image_pathname)
;					init_bad = 0
;				endif else begin
;					MSP_BDP_print_text, Event, 'Found non-first Image with Bad LL Quad = '+FILE_BASENAME(image_pathname)
;					init_bad = 1
;				endelse
;				;
;				;	fix the image - can't fix raw1 images easily so just set image to bad and skip
;				;
;				(*n_ptr).image_type[ind[0]] = '**BAD**'
;				;
;				;fx_status = lower_left_quadrant_repair(ll_ptr, init_bad)
;				;
;			endif
			;
;			;	return the heap space
;			;
;			PTR_FREE, ll_ptr
;			;
;			image_type = (*n_ptr).image_type[ind[0]]
;			if(image_type eq "**BAD**" or STRUPCASE(image_type) eq "SKIP") then begin
;				MSP_BDP_print_text, Event, "Image number "+STRING(format='(I)',image_number)+" is BAD or skip-designated, so skipping"
;				goto, image_loop_end
;			endif
			;
			;	Kokopeli analysis call
			;
			MSP_BDP_P3_kokepeli_analysis, Event, image_pathname, shift_struct, ierr
			if(ierr ne 0) then begin
				MSP_BDP_print_text, Event, "Skipping kokopeli analysis of file = "+FILE_BASENAME(image_pathname)
				goto, image_loop_end
			endif
			;
			;	evaluate results
			;
			if( shift_struct.k_shift eq 0) then begin
				MSP_BDP_print_text, Event, "No shifts found for image = "+FILE_BASENAME(image_pathname)
				goto, image_loop_end
			endif
			;
			MSP_BDP_print_text, Event, "Potential shifts found for image = "+FILE_BASENAME(image_pathname)
			;
			;	shift found, populate shift structure
			;
			ii = need_shift_counter
			(*k_ptr)[ii].night = night_desig
			(*k_ptr)[ii].group = group_id
			(*k_ptr)[ii].image = image_number
			(*k_ptr)[ii].image_pathname = image_pathname
			(*k_ptr)[ii].LL = shift_struct.LL
			(*k_ptr)[ii].LR = shift_struct.LR
			(*k_ptr)[ii].UL = shift_struct.UL
			(*k_ptr)[ii].UR = shift_struct.UR
			(*k_ptr)[ii].sLL = shift_struct.sLL
			(*k_ptr)[ii].sLR = shift_struct.sLR
			(*k_ptr)[ii].sUL = shift_struct.sUL
			(*k_ptr)[ii].sUR = shift_struct.sUR
			(*k_ptr)[ii].recommend = 'None'
			;
			need_shift_counter++
			;
			;	if no shift, skip to end
			; ---------------
			;
			image_loop_end:
			;
			;	update need shift total
			;
			need_shift_text = STRING(format='(I5.1)',need_shift_counter)
			WIDGET_CONTROL, (*s_ptr).P3_window_wid[2], SET_VALUE=need_shift_text
			;
			this_LED = ROUND(float(image_counter-1) / LED_scale)
			this_LED = 0 > ((n_LED-1) < this_LED)
			(*s_ptr).P3_LED_colors[this_LED] = "green"
			WCT_LED_stack, (*s_ptr).P3_LED_wid[0], (*s_ptr).P3_LED_colors
			;
			;	test for column repeat problem - added 20091013 DPC, using MP code
			;
			if( (*s_ptr).P3_column_flag ) then begin
				col_flag = P3_detect_clocking_errors(image_pathname)
				if(col_flag[0]+col_flag[3]+col_flag[6]+col_flag[9] gt 0) then begin
					MSP_BDP_print_text, Event, "Found col clock error in image "+FILE_BASENAME(image_pathname)
					(*c_ptr)[n_bad_col].night = night_desig
					(*c_ptr)[n_bad_col].group = group_id
					(*c_ptr)[n_bad_col].image = image_number
					(*c_ptr)[n_bad_col].image_pathname = image_pathname
					(*c_ptr)[n_bad_col].col_code[*] = col_flag[*]
					n_bad_col++
					;
					WIDGET_CONTROL, (*s_ptr).P3_window_wid[3], SET_VALUE=STRING(n_bad_col, format='(i4)')
				endif
			endif
		endfor
		;
		group_end:
		;
	endfor
	;
	;	if the number of images with column clocking errors is non-zero, start
	;	repair
	;
	if(n_bad_col gt 0) then begin
		(*s_ptr).num_column = n_bad_col
		;
		;	for each of the images, read the image, create mask, and write out
		;		to a subdirectory of Kokopeli
		;
		for i_col = 0, n_bad_col - 1 do begin
			;
			image_pathname = (*c_ptr)[i_col].image_pathname
			image_mask = P3_build_clocking_error_mask((*c_ptr)[i_col].col_code)	; 1024x1026 mask of 1, 0
			;
			;	does subdirectory exist?
			;
			reduced_dir = (*s_ptr).root_reduced
			night_desig = 	(*c_ptr)[i_col].night
			this_night_text = STRING(format='(I8.8)',night_desig)
			koko_night_dir = fix_path( reduced_dir + '3_Kokopeli_Repaired_Images' + PATH_SEP() + 'Col_Fix' + PATH_SEP() + this_night_text )
			;
			test = FILE_INFO(koko_night_dir)
			if(test.exists ne 1 or test.directory ne 1) then begin
				MSP_BDP_print_text, Event, "Creating Col_Fix subdirectory in Kokopeli directory"
				FILE_MKDIR, koko_night_dir
			endif
			;
			out_path = fix_path( koko_night_dir + PATH_SEP() + FILE_BASENAME(image_pathname) )
			writefits, out_path, image_mask
			;
			MSP_BDP_print_text, Event, "Wrote Col Mask file for image "+FILE_BASENAME(image_pathname)
		endfor
	endif
	;
	;	if the number of shifts needed is non-zero, turn on the next two buttons
	;
	if(need_shift_counter ne 0) then begin
		(*s_ptr).num_kokopeli = need_shift_counter
		(*s_ptr).fix_kokopeli = 0
		P3_button_sensitize, Event, 1, 1
		P3_button_sensitize, Event, 2, 1
	endif else begin
		;
		;	or write a message if no shifts
		;
		(*s_ptr).num_kokopeli = 0
		(*s_ptr).fix_kokopeli = 0
		MSP_BDP_print_text, Event, "This Night has no Kokopeli Shifts. No need to fix."
		;
		; *** check, fix col clock errors
		;
		if( (*s_ptr).num_column gt 0) then begin
			MSP_BDP_print_text, Event, "In Start-Kokopeli, checking, fixing column clock errors"
			P3_fix_remaining_clock_errors, Event
		endif
		;
		;	clear buttons and LEDs
		;
		P3_button_sensitize, Event, 0, 0
		P3_button_sensitize, Event, 1, 0
		P3_button_sensitize, Event, 2, 0
		P3_button_sensitize, Event, 3, 0
		P3_button_sensitize, Event, 4, 0
		;
		P3_LED_sensitize, Event, 0, 0
		P3_LED_sensitize, Event, 1, 0
	endelse
	;
	;	save the night file summary, to pick up turn off of directory pointers
	;
	MSP_BDP_Create_Night_File_Path, Event, night_desig, night_file_path
	MSP_BDP_Save_Night_File, Event, night_file_path
	;
	P3_Save_Night_Summary, Event
	;
end
;
;-----------------------------------------------------------------
pro P3_View_Edit_Shifts, Event
	;
	;	display table of shift summaries and recommendations
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	;
	;	build the table structure
	;
	col_headers = ["Image","Group","Grp.Type", "Exp. T", " LL", " LR", " UL",$
					" UR", "Grades", "Recommend"]
	;
	n_shifts = (*s_ptr).num_kokopeli
	format = strarr(10,n_shifts)
	format[0,*] = '(i4.3)'
	format[1,*] = '(i3.1)'
	format[2,*] = '(a)'
	format[3,*] = '(f6.2)'
	format[4:7,*] = '(I3.1)'
	format[8,*] = '(a4)'
	format[9,*] = '(a)'
	;
	color_array = strarr(10,n_shifts)
	;
	row_headers = strarr(n_shifts)
	data_line = {image: 0L, group: 0, grp_type: '', expt: 0.0, LL: 0, LR: 0,$
				 UL: 0, UR: 0, grades: '', recommend: ''}
	data_array = REPLICATE(data_line, n_shifts)			; array of structures
	;
	need_fix = 0L
	;
	for ii = 0, n_shifts-1 do begin
			;
			;	load structure for this image
			;
			row_headers[ii] = STRING(format='("K_",i3.3)',ii+1)
			;
			data_array[ii].image = (*k_ptr)[ii].image
			group_id = (*k_ptr)[ii].group
			data_array[ii].group = group_id + 1
			;
			;	go get the exposure time, group type for this image
			;
			expt = (*n_ptr).groups[group_id].group_exptime
			type = (*n_ptr).groups[group_id].group_type
			;
			data_array[ii].expt = expt
			data_array[ii].grp_type  = type
			data_array[ii].LL    = (*k_ptr)[ii].LL
			data_array[ii].LR    = (*k_ptr)[ii].LR
			data_array[ii].UL 	 = (*k_ptr)[ii].UL
			data_array[ii].UR    = (*k_ptr)[ii].UR
			;
			;	pack grades
			;
			grades = STRING(format='(a1,a1,a1,a1)', (*k_ptr)[ii].sLL, $
													(*k_ptr)[ii].sLR, $
													(*k_ptr)[ii].sUL, $
													(*k_ptr)[ii].sUR )
			data_array[ii].grades = grades
			;
			;	examine shifts in detail - if not "normal" then flag them
			;
			case data_array[ii].LL of
				0:	color_array[4,ii] = 'green'
				-1:	color_array[4,ii] = 'yellow'
				else: color_array[4,ii] = 'red'
			endcase
			;
			case data_array[ii].LR of
				0:	color_array[5,ii] = 'green'
				1:	color_array[5,ii] = 'yellow'
				else: color_array[5,ii] = 'red'
			endcase
			;
			case data_array[ii].UL of
				0:	color_array[6,ii] = 'green'
				-1:	color_array[6,ii] = 'yellow'
				else: color_array[6,ii] = 'red'
			endcase
			;
			case data_array[ii].UR of
				0:	color_array[7,ii] = 'green'
				1:	color_array[7,ii] = 'yellow'
				else: color_array[7,ii] = 'red'
			endcase
			;
			odd_flag = 0
			for id = 4, 7 do if( color_array[id,ii] eq 'red') then odd_flag = 1
			;
			;
			;	form recommendation
			;
			;	if already allowed answer, leave alone
			;
			case (*k_ptr)[ii].recommend of
				'Fix':	r_flag = 0
				'Maybe Ignore': r_flag = 0
				"Ignore": r_flag = 0
				"Odd, Ignore": r_flag = 0
				else: r_flag = 1
			endcase
			;
			if(r_flag eq 1) then begin
				;
				;		if BIAS type, recommend not FIX
				;
				;		if exposure time < 0.1 sec also recommend not FIX
				;
				recommend = "Fix"

				if( type eq "Bias" or (type eq "Dark" and expt lt 0.1)) then recommend = "Ignore" else begin
					if( expt lt 0.4) then recommend = "Maybe Ignore"
				endelse
				;
				if( odd_flag ) then recommend = "Odd, Ignore"
				;
				;
				data_array[ii].recommend    = recommend
				(*k_ptr)[ii].recommend = recommend
			endif else begin
				data_array[ii].recommend = (*k_ptr)[ii].recommend
				recommend = (*k_ptr)[ii].recommend
			endelse
			;
			color_array[0:3,ii] = "white"
			;
			case recommend of
				"Fix": color = "red"
				"Ignore": color = "goldenrod light"
				"Maybe Ignore": color = "orange"
				"Odd, Ignore": color = "yellow"
				else: color = "white"
			endcase
			;
			color_array[9,ii] = color
			;
			case type of
				"Skip": color = 'red'
				"**BAD**":	color = "red"
				"Bias":	color = "skyblue"
				"Dark": color = "grey68"
				"Pol-Img": color = "lawngreen"
				"Pol-Spec": color = "pink"
				"Spec": color = "plum"
				"Image": color = "yellow"
				"S-Argon": color = "slateblue"
				"Lin-On": color = "khaki light"
				"Lin-Off": color = "khaki dark"
				"I-Flat-On": color = "goldenrod light"
				"I-Flat-Off": color = "goldenrod dark"
				"S-Flat-On": color = "rosybrown light"
				"S-Flat-Off": color = "rosybrown dark"
				"P-Flat-On": color = "palegreen light"
				"P-Flat-Off": color = "palegreen dark"
				else: color = "white"
			endcase
			;
			color_array[0:3,ii] = color
			color_array[8,ii] = color
			if( odd_flag ) then color_array[8,ii] = 'red'
			;
	endfor
	;
	r_string = STRING(format='("Non-Zero Kokopeli Crack Shifts for Night ",I8.8)', (*n_ptr).night_desig)
	new_struct = Dialog_Big_Table(Title="Kokopeli Shift Images"+$
		'      can override recommendation by changing "Recommend" entry',Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
	;
	;	load new table data back into structure
	;
	nc = N_TAGS(new_struct)
	nr = N_ELEMENTS(new_struct)
	;
	;
	for ir = 0, nr-1 do begin
		(*k_ptr)[ir].group = new_struct[ir].group - 1
		(*k_ptr)[ir].recommend = new_struct[ir].recommend
		;
		;	count the FIX recommendations
		;
		do_fix = STRUPCASE(STRTRIM( (*k_ptr)[ir].recommend,2) )
		if( do_fix eq "FIX") then need_fix++
		;
		(*k_ptr)[ir].LL = new_struct[ir].LL
		(*k_ptr)[ir].LR = new_struct[ir].LR
		(*k_ptr)[ir].UL = new_struct[ir].UL
		(*k_ptr)[ir].UR = new_struct[ir].UR
	endfor
	;
	;	if need_fix ne 0, turn on next buttons
	;
	if(need_fix ne 0) then begin
		;
		(*s_ptr).fix_kokopeli = need_fix
		;
		P3_button_sensitize, Event, 3, 1
		P3_button_sensitize, Event, 4, 1
		P3_LED_sensitize, Event, 1, 1
	endif else begin
		;
		;	otherwise, set step to 3 for this night and resave
		;
		;****** check, fix col clock errors
		;
		if( (*s_ptr).num_column gt 0) then begin
			MSP_BDP_print_text, Event, "In View-Edit-Shifts, checking, fixing column clock errors"
			P3_fix_remaining_clock_errors, Event
		endif
		;
		P3_Save_Night_Summary, Event
		;
		P3_button_sensitize, Event, 0, 0
		P3_button_sensitize, Event, 1, 0
		P3_button_sensitize, Event, 2, 0
		P3_button_sensitize, Event, 3, 0
		P3_button_sensitize, Event, 4, 0
		;
		P3_LED_sensitize, Event, 0, 0
		P3_LED_sensitize, Event, 1, 0
		;
		;
	endelse
end

;-----------------------------------------------------------------
pro P3_Zero_Shifts, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	;
	(*s_ptr).fix_kokopeli = 0		; turn off shifts
	(*s_ptr).num_kokopeli = 1
	;
	;
	;****** check, fix col clock errors
	;
	if( (*s_ptr).num_column gt 0) then begin
		MSP_BDP_print_text, Event, "In Zero-Shifts, checking, fixing column clock errors"
		P3_fix_remaining_clock_errors, Event
	endif
	;
	P3_Save_Night_Summary, Event
	;
	P3_button_sensitize, Event, 0, 0
	P3_button_sensitize, Event, 1, 0
	P3_button_sensitize, Event, 2, 0
	P3_button_sensitize, Event, 3, 0
	P3_button_sensitize, Event, 4, 0
	;
	P3_LED_sensitize, Event, 0, 0
	P3_LED_sensitize, Event, 1, 0
	;
	MSP_BDP_print_text, Event, "Zeroing Shifts -- Done with this night."
end

;-----------------------------------------------------------------
pro P3_Ignore_Shifts, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	;
	(*s_ptr).fix_kokopeli = 0
	(*s_ptr).num_kokopeli = 1
	;
	;****** check, fix col clock errors
	;
	if( (*s_ptr).num_column gt 0) then begin
		MSP_BDP_print_text, Event, "In Ignore-Shifts, checking, fixing column clock errors"
		P3_fix_remaining_clock_errors, Event
	endif
	;
	P3_Save_Night_Summary, Event
	;
	P3_button_sensitize, Event, 0, 0
	P3_button_sensitize, Event, 1, 0
	P3_button_sensitize, Event, 2, 0
	P3_button_sensitize, Event, 3, 0
	P3_button_sensitize, Event, 4, 0
	;
	P3_LED_sensitize, Event, 0, 0
	P3_LED_sensitize, Event, 1, 0
	;
	MSP_BDP_print_text, Event, "Ignoring Shifts -- Done with this night."
end

;-----------------------------------------------------------------
pro P3_Do_Shifts, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	;
	reduced_dir = (*s_ptr).root_reduced
	;
	;	for each image needing repair, do the repair, putting image into
	;	the P3_kokopeli subdirectory off the main reduced data directory
	;
	;	the subprogram deals with all the directory creation itself
	;
	num_fix = 		(*s_ptr).fix_kokopeli		; the number of images to fix
	num_total = 	(*s_ptr).num_kokopeli	; the number of images on the full list (includes ignores)
	;
	num_images = 	(*n_ptr).num_images
	;
	night_desig = 	(*k_ptr)[0].night
	this_night_text = STRING(format='(I8.8)',night_desig)
	koko_night_dir = '3_Kokopeli_Repaired_Images' + PATH_SEP() + this_night_text
	;
	;	populate the next level of cds/raw1 directory buffers
	;
	output_CDS_dir = fix_path( koko_night_dir + PATH_SEP() + "CDS_RAW" + PATH_SEP() )
	output_RAW1_dir = fix_path( koko_night_dir + PATH_SEP() + "RAW1_RAW" + PATH_SEP() )
	(*n_ptr).paths_cds_root[1] = output_CDS_dir
	(*n_ptr).paths_raw1_root[1] = output_RAW1_dir
	;
	i_fix = 0
	;
	;	clear LED progress stack to orange
	;
	n_LED = N_ELEMENTS((*s_ptr).P3_LED_colors)
	(*s_ptr).P3_LED_colors[*] = "yellow"
	;
	LED_scale = float(num_fix) / float(n_LED)
	if(LED_scale lt 1.0) then LED_scale = 1.0
	if(num_fix lt n_LED) then (*s_ptr).P3_LED_colors[num_fix:n_LED-1] = 'black'
	WCT_LED_stack, (*s_ptr).P3_LED_wid[1], (*s_ptr).P3_LED_colors
	;
	;----------
	;
	image_counter = 0L
	last_LED = 0
	;
	;****** check, fix col clock errors
	;
	if( (*s_ptr).num_column gt 0) then begin
		MSP_BDP_print_text, Event, "In Do-Shifts, checking, fixing column clock errors"
		P3_fix_remaining_clock_errors, Event
	endif
	;
	;
	for ik = 0, num_total-1 do begin
		;
		;	if not of type "Fix" then skip
		;
		if( (*k_ptr)[ik].recommend eq "Fix" ) then begin
			;
			;	otherwise need to fix this image
			;
			shifts = intarr(4)
			shifts[0] = (*k_ptr)[ik].LL
			shifts[1] = (*k_ptr)[ik].LR
			shifts[2] = (*k_ptr)[ik].UL
			shifts[3] = (*k_ptr)[ik].UR
			;
			input_CDS_pathname = (*k_ptr)[ik].image_pathname
			;
			;	if this pathname is not matched in the list of clock error pathnames, go ahead
			;	and get data from raw directories. But, if matched, then fetch from Koko directories
			;
			if( SIZE(c_ptr,/TYPE) eq 0) then begin
				nmatch = 0
			endif else begin
				ind_match = WHERE( FILE_BASENAME( (*c_ptr)[*].input_pathname) eq FILE_BASENAME(input_CDS_pathname), nmatch)
			endelse
			if(nmatch eq 1) then begin
				;
				;	matched names, so images should be obtained from Kokopeli *output* directories
				;
				night_desig = 	(*c_ptr)[ind_match].night
				this_night_text = STRING(format='(I8.8)',night_desig)
				input_CDS_pathname = fix_path( (*s_ptr).root_reduced + koko_night_dir + PATH_SEP() + "CDS_RAW" + PATH_SEP() + FILE_BASENAME(input_CDS_pathname) )
			endif
			;
			;	call the engine that fixes the images - pass it the output CDS directory name
			;
			MSP_BDP_Make_sky_true, Event, shifts, input_CDS_pathname, output_CDS_dir
			;
			i_fix++
			;
			;	set the image_root_index value for this image to 1 to signify it
			;	was changed by kokopeli analysis
			;
			;  *** and update night image_root_index pointer to point to 1 (koko corrected)
			;
			image_desig = (*k_ptr)[ik].image
			;
			ind = WHERE( abs((*n_ptr).image_list[0:num_images-1]) eq image_desig, count)
			if(count eq 1) then begin
				(*n_ptr).image_root_index[ind[0]] = 1L
				;
;				; which group?
;				;
;				group = (*k_ptr)[ik].group
			endif else begin
				MSP_BDP_print_text, Event, "ERROR - can't match image desig in Do-Shifts"
				print, "Image_desig = ",image_desig
				print, "  Match count = ",count
				print, "  Num_images = ",num_images
				print, "  IK = ",ik
				print, "  Image list = ", (*n_ptr).image_list[0:num_images-1]
			endelse
			;
			this_LED = ROUND(float(i_fix) / LED_scale)
			this_LED = 0 > ((n_LED-1) < this_LED)
			(*s_ptr).P3_LED_colors[this_LED] = "green"
			WCT_LED_stack, (*s_ptr).P3_LED_wid[1], (*s_ptr).P3_LED_colors
		endif
	endfor
	;
	;	did we fix them all?
	;
	if(num_fix ne i_fix) then MSP_BDP_print_text, Event, "Whoops! Mismatch for kokopeli fix images"
	;
	;	rewrite the night summary
	;
	MSP_BDP_Create_Night_File_Path, Event, night_desig, night_file_path
	MSP_BDP_Save_Night_File, Event, night_file_path
	;
	P3_Save_Night_Summary, Event
	;
	MSP_BDP_print_text, Event, "Corrected all images needing shifting for night = "+this_night_text
	;
	;	shut down all buttons and LEDs
	;
	P3_button_sensitize, Event, 0, 0
	P3_button_sensitize, Event, 1, 0
	P3_button_sensitize, Event, 2, 0
	P3_button_sensitize, Event, 3, 0
	P3_button_sensitize, Event, 4, 0
	;
	P3_LED_sensitize, Event, 0, 0
	P3_LED_sensitize, Event, 1, 0
end
;
;-----------------------------------------------------------------
pro P3_Save_Night_Summary, Event
	;
	;	saves and updated night summary, based on properties
	;	of the number of shifts recommended or completed
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	;
	;	how may kokopeli shifts were found, and/or recommended as FIX?
	;
	n_found = (*s_ptr).num_kokopeli
	n_fix = (*s_ptr).fix_kokopeli
	;
	this_night = (*n_ptr).night_desig
	;
	;	which one in the summary?
	;
	num_nights = (*s_ptr).num_nights
	ind = WHERE(this_night eq (*s_ptr).night_date[0:num_nights-1], count)
	if(count ne 0) then night_index = ind[0] else begin
		MSP_BDP_print_text, Event, "Error - No matching night in P3..."
		goto, P3SNS_end
	endelse
	;
	(*s_ptr).night_who[night_index] = (*s_ptr).who
	if(n_found eq 0) then begin
		(*s_ptr).night_step[night_index] = 3
		(*s_ptr).night_comment[night_index] = "No Shifts"
	endif else begin
		if(n_fix eq 0) then begin
			(*s_ptr).night_step[night_index] = 3
			(*s_ptr).night_comment[night_index] = "Shifts Ignored"
		endif else begin
			;
			;	if n_fix not equal to zero, fixed them
			;
			(*s_ptr).night_step[night_index] = 3
			(*s_ptr).night_comment[night_index] = "Shifts Fixed"
		endelse
	endelse
	;
	P1_Write_Summary_File, Event
	;
	P3SNS_end:
end
;-----------------------------------------------------------------
pro P3_fix_remaining_clock_errors, Event
	;
	;	looks at the c_struct to find images with non-zero column
	;	clocking errors, fetches those images from the CDS & RAW1 directories
	;	fixes them and puts them into the Kokopeli directories for later
	;	use. Also flags night structure that the images are in the Koko dir.
	;
	;	DPC	20091014
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	c_ptr = PTR_struct.PTR_Column
	;
	reduced_dir = (*s_ptr).root_reduced
	;
	night_desig = (*n_ptr).night_desig
	;this_night_text = STRING(format='(I8.8)',night_desig)
	;
	;	max number to check?
	;
	max_num = (*s_ptr).num_column
	;
	;	so, check each - do we need to fix, or was fix already done?
	;
	for i_image = 0, max_num-1 do begin
		;
		this_image_number = (*c_ptr)[i_image].image
		;
		if(this_image_number ne -1) then begin			; -1 means "already fixed"
			;
			;	rebuild pathname to input CDS file
			;
			night_desig = 	(*c_ptr)[i_image].night
			this_night_text = STRING(format='(I8.8)',night_desig)
			koko_night_dir = '3_Kokopeli_Repaired_Images' + PATH_SEP() + this_night_text
			;
			;	populate the next level of cds/raw1 directory buffers
			;
			output_CDS_dir = fix_path( koko_night_dir + PATH_SEP() + "CDS_RAW" + PATH_SEP() )
			output_RAW1_dir = fix_path( koko_night_dir + PATH_SEP() + "RAW1_RAW" + PATH_SEP() )
			(*n_ptr).paths_cds_root[1] = output_CDS_dir
			(*n_ptr).paths_raw1_root[1] = output_RAW1_dir
			;
			this_image_path = (*c_ptr)[i_image].image_pathname
			;
			;	call the engine that fixes the images - pass it the output CDS directory name
			;
			P3_mask_columns_and_save, Event, this_image_number, output_CDS_dir
			;
			;	set the image_root_index value for this image to 1 to signify it
			;	was changed by kokopeli analysis
			;
			;  and update night image_root_index pointer to point to 1 (koko corrected)
			;
			image_desig = (*c_ptr)[i_image].image
			;
			ind = WHERE( abs((*n_ptr).image_list[0:(*n_ptr).num_images-1]) eq image_desig, count)
			if(count eq 1) then begin
				(*n_ptr).image_root_index[ind[0]] = 1L
			endif
			;
			;	clear this image off the clock list
			;
			(*c_ptr)[i_image].image = -1
			;
			;	test next image on list
			;
		endif
	endfor
	;
	;	rewrite the night summary
	;
	MSP_BDP_Create_Night_File_Path, Event, night_desig, night_file_path
	MSP_BDP_Save_Night_File, Event, night_file_path
	;
	MSP_BDP_print_text, Event, "Finished repairing remaining images with column clock errors"
end
;
;---------------------------------------------------------------------------------------
;
pro P3_mask_columns_and_save, Event, input_image_number, output_CDS_dir
	;
	;	for one image, fetches mask image for column clock error and applies
	;	it to the CDS and RAW1 images and then stores them in the output directory
	;
	;	DPC 20091014
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	c_ptr = PTR_struct.PTR_Column
	;
	reduced_dir = (*s_ptr).root_reduced
	;
	ind_image = WHERE( (*n_ptr).image_list[*] eq input_image_number, nmatch)
	if(nmatch eq 1) then begin
		input_CDS_pathname = get_CDS_pathname(Event, ind_image[0])
		;
		input_RAW1_pathname = get_RAW1_pathname(Event, ind_image[0])
		;
		CDS_image = readfits(input_CDS_pathname, CDS_header, /SILENT)
		CDS_size = SIZE(CDS_image)
		xsize = CDS_size[1]
		ysize = CDS_size[2]
		if(xsize eq 1024 and ysize eq 1026) then begin
			RAW1_image = read_raw1(input_RAW1_pathname)
		endif else begin
			RAW1_image = read_raw1(input_RAW1_pathname, xsize, ysize)
		endelse
		;
		;	get mask
		;
		reduced_dir = (*s_ptr).root_reduced
		;
		ind = WHERE( (*c_ptr).image eq input_image_number, n_night)
		if(n_night eq 1) then begin
			night_desig = 	(*c_ptr)[ind].night
			image_pathname = (*c_ptr)[ind].image_pathname
			this_night_text = STRING(format='(I8.8)',night_desig)
			koko_night_dir = fix_path( reduced_dir + '3_Kokopeli_Repaired_Images' + $
							PATH_SEP() + 'Col_Fix' + PATH_SEP() + this_night_text )
			;
			mask_path = fix_path( koko_night_dir + PATH_SEP() + FILE_BASENAME(image_pathname) )
			;
			mask_image = readfits(mask_path,mask_header,/SILENT)
			;
			MSP_BDP_print_text, Event, "Fetched mask_image for "+FILE_BASENAME(image_pathname)
			;
			;	mask CDS pixels to deleted (-1.0e6) and RAW1 pixels to deleted (0)
			;
			for ix = 0, xsize-1 do begin
				mvec = REFORM(mask_image[ix,*])
				ind = WHERE(mvec eq 0, nmask)
				if(nmask gt 0) then begin
					CDS_image[ix,ind] = -1.0e6
					RAW1_image[ix,ind] = 0
				endif
			endfor
			;
			;	write images back into Kokopeli CDS, RAW1 subdirectories
			;
			SXADDPAR, CDS_header, "HISTORY","Masked columns with clocking error - BDP S3+"
			output_CDS_path = fix_path( reduced_dir + output_CDS_dir + PATH_SEP() + FILE_BASENAME(image_pathname) )
			;
			;	check that night directory already present. If not, create
			;
			output_night_dir = FILE_DIRNAME(output_CDS_path)
			test = FILE_INFO(output_night_dir)
			if(test.exists ne 1 or test.directory ne 1) then begin
				MSP_BDP_print_text, Event, "Creating new directory: "+output_night_dir
				FILE_MKDIR, output_night_dir
			endif
			;
			writefits, output_CDS_path, CDS_image, CDS_header
			;
			output_RAW1_path = CDS_to_RAW1_pathname(output_CDS_path)
			output_night_dir = FILE_DIRNAME(output_RAW1_path)
			test = FILE_INFO(output_night_dir)
			if(test.exists ne 1 or test.directory ne 1) then begin
				MSP_BDP_print_text, Event, "Creating new directory: "+output_night_dir
				FILE_MKDIR, output_night_dir
			endif
			pix = write_raw1(output_RAW1_path, RAW1_image)
			;
			MSP_BDP_print_text, Event, "Wrote new (masked) versions of CDS, RAW1 images for "+FILE_BASENAME(image_pathname)
			;
		endif else begin
			MSP_BDP_print_text, Event, "ERROR - could not match to image number "+STRING(input_image_number)
		endelse
	endif else begin
		MSP_BDP_print_text, Event, "ERROR - no match for image number "+STRING(input_image_number)
	endelse
;
end
