;
;	Part 4 Event Handler Routines - RAW1 Jump Finder/Fixer
;
;-----------------------------------------------------------------
;
pro P4_Select_Night, Event
	;
	MSP_BDP_print_text, Event, "Night Selection Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	;
	wid = (*s_ptr).P4_droplist_night_wid
	;
	night_index = Event.index
	;
	;	index is not enough, as we might not have done all the part 2
	;	stuff.... So, need to get the night designation from the list
	;
	WIDGET_CONTROL, wid, GET_VALUE = vector_night_desig
	selected_night_desig = vector_night_desig[night_index]
	;
	;print, "in droplist, index = ",night_index
	;
	;night_desig = (*s_ptr).night_date[night_index]
	night_desig = selected_night_desig
	night_text = STRING(format='(I8.8)',night_desig)
	;
	;	reset buttons and LEDs
	;
	for iw = 0, N_ELEMENTS( (*s_ptr).P4_button_wid)-1 do begin
		if(iw le 0) then sense = 1 else sense = 0
		P4_button_sensitize, Event, iw, sense
		if(iw le 1) then P4_LED_sensitize, Event, iw, sense
	endfor
	;
	;	read in that night's save file
	;
	MSP_BDP_print_text, Event, "Opening Night Summary for night = "+night_text
	;
	MSP_BDP_Create_Night_File_Path, Event, night_desig, night_file_path
	MSP_BDP_Restore_Night_File, Event, night_file_path, description, ierr
	;
	if(ierr ne 0) then begin
		MSP_BDP_print_text, Event, "Could not restore night "+night_text+" - skipping"
		goto, P4SN_end
	endif
	;
	;	must redo pointers
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	raw_root = (*s_ptr).root_raw
	;
	;	capture number of groups
	;
	ngroups = (*n_ptr).num_groups
	ngroups_text = STRING(format='(I)',ngroups)
	MSP_BDP_print_text, Event, "Restored "+night_text+" with "+ngroups_text+" groups"
	;
	;	clear deviation array
	;
	n_images = (*n_ptr).num_images
	(*r_ptr).deviation_RAW1[*,*,0:n_images-1] = 3000.0		; to make it obvious when it fails
	;
	;	turn on first button and progress bar
	;
	P4_button_sensitize, Event, 0, 1
	P4_LED_sensitize, Event, 0, 1
	;
	;	and clear plot
	;
	WSET, (*s_ptr).P4_draw_window_id
	ERASE
	;
	P4SN_end:
end
;
;------------------------------------------------------
pro P4_RAW1_Examination, Event
	;
	;	look at all the images and measure the mean counts
	;
	;	do using groups, but then sort by image number
	;
	MSP_BDP_print_text, Event, "Examination of RAW1 images for jumps requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	r_ptr = PTR_struct.PTR_RAW1
	;
	;	make the LEDs all orange
	;
	colors = strarr(25)
	colors[*] = "orange"
	WCT_LED_stack, (*s_ptr).P4_LED_wid[0], colors
	;
	;	build the input list of pathnames to RAW1 images
	;
	night_desig = 		(*n_ptr).night_desig
	night_desig_text = 	STRING(format='(I8.8)',night_desig)
	;
	root_raw = (*s_ptr).root_raw
	;
	n_images = 		(*n_ptr).num_images
	file_list = 	(*n_ptr).image_list[0:n_images-1]
	image_type = 	(*n_ptr).image_type[0:n_images-1]
	image_frame = 	(*n_ptr).image_frame[0:n_images-1]
	;
	image_coadds =  (*n_ptr).image_coadds[0:n_images-1]
	;
	input_pathname_list = strarr(n_images)
	x_plot = intarr(n_images)
	y_plot = fltarr(n_images)
	;
	jump_coadds = intarr(n_images)
	;
	n_input = 0L
	;
	;	must build input image list from the images listed for each group
	;	since un-grouped images are not to be processed
	;
	n_groups = (*n_ptr).num_groups
	;
	for ig = 0L, n_groups-1L do begin
		n_this_group = 	(*n_ptr).groups[ig].group_images
		group_list = 	(*n_ptr).groups[ig].group_list[0:n_this_group-1]
		;
		;	for every image in this group
		;
		for itg = 0, n_this_group-1 do begin
			image_no = group_list[itg]
			;
			ind = WHERE(image_no eq file_list[*],count)
			if(count ne 1) then begin
				MSP_BDP_print_text, Event, "ERROR - no match of group to image number for IG, Image_no = "+STRING(format='(i,1x,i)',ig,image_no)
				goto, ITG_loop_end
			endif
			;
			;	match found. save index into full image list
			;
			i = ind[0]
			;
			;	now, test image type and frame size
			;
			if( (image_type[i] ne "**BAD**") and (image_type[i] ne 'Skip') and $
				(image_frame[i] eq "Full") ) then begin
				;
				;	create RAW1 pathname
				;
				image_no_text = STRTRIM(STRING(format='(I5.3)',image_no),2)
				path = "raw1_" + image_no_text
				;
				;
				x_plot[n_input] = image_no
				;
				root_index = (*n_ptr).image_root_index[i]
				if(root_index eq 0) then begin
					base_dir = (*s_ptr).root_raw
				endif else begin
					base_dir = (*s_ptr).root_reduced
				endelse
				;
				root_dir = base_dir + (*n_ptr).paths_raw1_root[root_index]
				;
				input_pathname_list[n_input] = fix_path( FILEPATH(path,ROOT_DIR=root_dir) )
				;
				jump_coadds[n_input] = image_coadds[i]		; save coadd number
				;
				;	set the jump flags to zero
				;
				(*r_ptr).jump_flags[0,n_input] = image_no
				(*r_ptr).jump_flags[1,n_input] = 0
				;
				n_input++
			endif
			;
			ITG_loop_end:
		endfor
	endfor
	;
	;	sort by image number
	;
	x_plot = 	x_plot[0:n_input-1]
	ind = 		SORT(x_plot)
	x_plot = 	x_plot[ind]
	jump_flags = (*r_ptr).jump_flags[*,0:n_input-1]
	(*r_ptr).jump_flags[*,0:n_input-1] = jump_flags[*,ind]
	input_pathname_list = input_pathname_list[ind]
	jump_coadds = jump_coadds[ind]
	;
	if(n_input le 0) then begin
		MSP_BDP_print_text, Event, "Found No RAW1 Images to Examine for night = "+night_desig_text
		goto, P4RE_end
	endif
	;
	MSP_BDP_print_text, Event, "Found "+STRING(format='(i)',n_input)+" RAW1 images to examine"
	;
	input_pathname_list = input_pathname_list[0:n_input-1]
	;
	;	call the examination engine
	;
	MSP_BDP_RAW1_Examiner, Event, input_pathname_list, jump_coadds, output_struct
	;
	;	load the structure
	;
	n_ele = N_ELEMENTS(output_struct)
	;
	;	try to copy entire structure array at once
	;
	(*r_ptr).detect[0:n_ele-1] = output_struct[0:n_ele-1]
	;
	y_plot = output_struct[0:n_ele-1].mean_even[0]
	x_plot = x_plot[0:n_ele-1]
	;
	(*r_ptr).x_plot[0:n_ele-1] = x_plot[*]
	(*r_ptr).y_plot[0:n_ele-1] = y_plot[*]
	(*r_ptr).n_plot = n_ele
	;
	;	turn on the print and detect buttons
	;
	for ii = 1, 11 do P4_button_sensitize, Event, ii, 1
	;
	;	and put a plot of the LLE data into the window
	;
	device, decomposed=0
	WSET, (*s_ptr).P4_draw_window_id
	;
	xmax = max(x_plot)
	xmin = min(x_plot)
	ymax = max(y_plot)
	ymin = min(y_plot)
	;
	ymax = ymax + 0.05*(ymax-ymin)
	ymin = ymin - 0.05*(ymax-ymin)
	;
	(*r_ptr).xmax = xmax
	(*r_ptr).xmin = xmin
	(*r_ptr).ymax = ymax
	(*r_ptr).ymin = ymin
	;
	(*r_ptr).xmax0 = xmax
	(*r_ptr).xmin0 = xmin
	(*r_ptr).start_frame = xmin
	(*r_ptr).end_frame = xmax
	;
	(*r_ptr).position = [70,40,600,420]
	(*r_ptr).group_flag = 0
	;
	P4_RAW1_Plot, Event
	;
	MSP_BDP_print_text, Event, "RAW1 Examination of Night "+night_desig_text+" completed"
	;
	;	compute deviations for each group as default
	;
	P4_Compute_Default_Deviations, Event
	;
	P4RE_end:
end
;
;-----------------------------------------------------
pro P4_Compute_Default_Deviations, Event
	;
	;	need these in order to use the "J" option outside of S-E groups
	;
	MSP_BDP_print_text, Event, "Compute default deviations requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	r_ptr = PTR_struct.PTR_RAW1
	;
	;	get number of groups, images
	;
	n_groups = (*n_ptr).num_groups
	n_images = (*n_ptr).num_images
	;
	;	reserve some temporary room for the raw1 values
	;
	raw1_group_values = dblarr(n_groups, 4, 2)
	;
	;	for each non-BAD group, compute mfm mean and then each image's deviation from that
	;
	for ig = 0, n_groups-1 do begin
		;
		;	list of images in this group
		;
		n_grp_images = (*n_ptr).groups[ig].group_images
		group_type =   (*n_ptr).groups[ig].group_type
		if(group_type eq '**BAD**' or group_type eq 'Skip') then begin
			;
			raw1_group_values[ig,*,0] = 0.0
			raw1_group_values[ig,*,1] = 0.0
			;
			goto, group_end
		endif
		;
		image_index_list = lonarr(n_grp_images)
		;
		ign = 0L
		for iig = 0, n_grp_images-1 do begin
			;
			;	get image number and match to night image list
			;
			group_image_number = (*n_ptr).groups[ig].group_list[iig]
			ind = WHERE( group_image_number eq (*n_ptr).image_list[0:n_images-1],count)
			if(count ne 1) then begin
				MSP_BDP_print_text, Event, "ERROR - no image number match in compute default deviations"
				goto, group_end
			endif
			;
			;	load values array
			;
			if( (*n_ptr).image_type[ind[0]] eq '**BAD**' or $
				(*n_ptr).image_type[ind[0]] eq 'Skip' or $
				( (*n_ptr).image_frame[ind[0]] ne "Full") ) then begin			; skip BAD images, though they
			;	n_grp_images--											; ought not to be in groups
				goto, loop_end
			endif
			;
			image_index_list[ign] = ind[0]
			ign++
			;
			loop_end:
		endfor
		;
		;	have all the indices for this group, compute MFM means for each quadrant, O/E rows
		;
		if(ign gt 1) then begin
			for iquad = 0, 3 do begin
				vec_e = REFORM( (*r_ptr).detect[image_index_list[0:ign-1]].mean_even[iquad] )
				vec_o = REFORM( (*r_ptr).detect[image_index_list[0:ign-1]].mean_odd[iquad] )
				;
				zz_e = median_filtered_mean(vec_e)
				zz_o = median_filtered_mean(vec_o)
				;
				;	load values array
				;
				raw1_group_values[ig,iquad,0] = zz_e[0]
				raw1_group_values[ig,iquad,1] = zz_o[0]
			endfor
		endif else begin
			if(ign eq 1) then begin
				raw1_group_values[ig,*,0] = (*r_ptr).detect[image_index_list[0]].mean_even[*]
				raw1_group_values[ig,*,1] = (*r_ptr).detect[image_index_list[0]].mean_odd[*]
			endif else begin
				;
				;	this is the negative case -- shouldn't get here!
				;
				MSP_BDP_print_text, Event, "ERROR - got negative image count in compute default deviations"
			endelse
		endelse
		;
		group_end:
	endfor
	;
	;	then, for each image, compute the difference between the group means and its values
	;	and load into the deviation array
	;
	for ig = 0, n_groups-1 do begin
		;
		n_grp_images = (*n_ptr).groups[ig].group_images
		group_type = (*n_ptr).groups[ig].group_type
		if(group_type eq '**BAD**' or group_type eq 'Skip') then begin
			;
			goto, group2_end
		endif
		;
		image_index_list = lonarr(n_grp_images)
		;
		ign = 0L
		for iig = 0, n_grp_images-1 do begin
			;
			;	get image number and match to night image list
			;
			group_image_number = (*n_ptr).groups[ig].group_list[iig]
			ind = WHERE( group_image_number eq (*n_ptr).image_list[0:n_images-1],count)
			if(count ne 1) then begin
				MSP_BDP_print_text, Event, "ERROR - no image number match in compute default deviations [2]"
				goto, group2_end
			endif
			;
			;
			if( (*n_ptr).image_type[ind[0]] eq '**BAD**' or $
				(*n_ptr).image_type[ind[0]] eq 'Skip' or $
				( (*n_ptr).image_frame[ind[0]] ne "Full")) then begin	; skip BAD images, though they
			;	n_grp_images--											; ought not to be in groups
				goto, loop2_end
			endif
			;
			;	compute the deviations of the current image from its group means
			;
			for iquad = 0, 3 do begin
				this_mean_e = raw1_group_values[ig,iquad,0]
				this_mean_o = raw1_group_values[ig,iquad,1]
				;
				(*r_ptr).deviation_RAW1[0,iquad,ind[0]] = (*r_ptr).detect[ind[0]].mean_even[iquad] - this_mean_e
				(*r_ptr).deviation_RAW1[1,iquad,ind[0]] = (*r_ptr).detect[ind[0]].mean_odd[iquad]  - this_mean_o
			endfor
			;
			loop2_end:
		endfor
		;
		group2_end:
	endfor
	;
	MSP_BDP_print_text, Event, "Default Deviations computed"
end
;
;-------------
pro P4_Print_Summary, Event
	;
	;	prints the same plot to selected printer
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	r_ptr = PTR_struct.PTR_RAW1
	;
	night_desig = (*n_ptr).night_desig
	night_desig_text = STRING(format='(I8.8)',night_desig)
	;
	n_pts = (*r_ptr).n_plot
	x_plot = (*r_ptr).x_plot[0:n_pts-1]
	y_plot = (*r_ptr).y_plot[0:n_pts-1]
	;
	;	get current plot device, for return after printing
	;
	plot_device = !D.NAME
	;
	set_plot, "PRINTER"
	zero = dialog_printersetup()
	Device, Landscape=1
	;
	xmax = max(x_plot)
	xmin = min(x_plot)
	ymax = max(y_plot)
	ymin = min(y_plot)
	;
	ymax = ymax + 0.05*(ymax-ymin)
	ymin = ymin - 0.05*(ymax-ymin)
	;
	plot, x_plot, y_plot, XRANGE=[xmin,xmax], YRANGE=[ymin,ymax], XSTYLE=1, YSTYLE=1, XTITLE="Image Number", $
    	Title="LLE RAW1 Mean Counts for night"+night_desig_text, XTHICK=4, YTHICK=3
	;
	Device, /Close_Document
	;
	;
	SET_PLOT, plot_device		; return to normal window plotting
end
;
;-------------------------------------------
pro P4_RAW1_Plot, Event
	;
	;	basic plotter
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	r_ptr = PTR_struct.PTR_RAW1
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	;
	night_desig = (*n_ptr).night_desig
	night_desig_text = STRING(format='(I8.8)',night_desig)
	;
	;   for the LL_E data, plot run of RAW1 with frame number
	;
	device, decomposed=0
	;
	n_pts = (*r_ptr).n_plot
	x_plot = (*r_ptr).x_plot[0:n_pts-1]
	y_plot = (*r_ptr).y_plot[0:n_pts-1]
	;
	xmax = (*r_ptr).xmax
	xmin = (*r_ptr).xmin
	ymax = (*r_ptr).ymax
	ymin = (*r_ptr).ymin
	;
	xmax0 = (*r_ptr).xmax0
	xmin0 = (*r_ptr).xmin0
	;
	; mark start, end frame numbers
	;
	start_frame = (*r_ptr).start_frame
	end_frame = (*r_ptr).end_frame
	;
	;	load group_plot_flag
	;
	group_flag = (*r_ptr).group_flag
	;
	;	load color lookup table
	;
	color_red = REFORM((*r_ptr).color_rgb[0,*])
	color_green = REFORM((*r_ptr).color_rgb[1,*])
	color_blue = REFORM((*r_ptr).color_rgb[2,*])
	;
	tvlct, color_red, color_green, color_blue, 0
	;
replot:
	plot, x_plot, y_plot, XRANGE=[xmin, xmax], YRANGE=[ymin, ymax], $
		XSTYLE=1, color=1, background=0, thick=1, YSTYLE=1, LINESTYLE=0, $
		Xtitle = "Image Number", TITLE="LLE RAW1 Mean Counts for night"+night_desig_text, $
		POSITION=(*r_ptr).position,/DEVICE, /NODATA
	;
	;	if group flag set, paint group colors as background
	;
	if(group_flag) then begin
		;
		;	for every group in the night, establish if this group is on the plot
		;
		num_groups = (*n_ptr).num_groups
		;
		for ig = 0, num_groups-1 do begin
			;
			;	fetch start, stop image/frame numbers for this group
			;
			group_start = (*n_ptr).groups[ig].group_start
			if((group_start-0.5) gt xmax) then goto, not_this_group
			group_end = (*n_ptr).groups[ig].group_end
			if((group_end+0.5) lt xmin) then goto, not_this_group
			;
			;	ok, some frame is in this zone
			;
			group_type = (*n_ptr).groups[ig].group_type
			; set paint color
			case group_type of
				"Skip": color = 'red'
				"**BAD**":	color = "red"
				"Bias":	color = "skyblue"
				"Dark": color = "grey68"
				"Pol-Img": color = "lawngreen"
				"Pol-Spec": color = "pink"
				"Spec": color = "plum"
				"Image": color = "yellow"
				"S-Argon": color = "slateblue"
				"Lin-On": color = "khaki light"
				"Lin-Off": color = "khaki dark"
				"I-Flat-On": color = "goldenrod light"
				"I-Flat-Off": color = "goldenrod dark"
				"S-Flat-On": color = "rosybrown light"
				"S-Flat-Off": color = "rosybrown dark"
				"P-Flat-On": color = "palegreen light"
				"P-Flat-Off": color = "palegreen dark"
				"R-Flat-Off": color = "steelblue dark"
				"R-Flat-On": color = "steelblue light"
				"PCal-On": color = "palegreen light"
				"PCal-Off": color = "palegreen dark"
				else: color = "black"
			endcase
			;
			;	for every frame in this group, paint a polygon at each frame,
			;	where the polygon color relates to group type and the offset
			;	vertically relates to the group ID number
			;
			n_this_group = (*n_ptr).groups[ig].group_images
			group_list = (*n_ptr).groups[ig].group_list[0:n_this_group-1]
			;
			yscale = 0.4*(ymax-ymin) / float(num_groups)
			dy = 0.6*(ymax-ymin)
			;
			for ifr = 0, n_this_group-1 do begin
				iframe = group_list[ifr]
				if((iframe+0.5) lt xmin) then goto, loop_end
				if((iframe-0.5) gt xmax) then goto, loop_end
				;
				this_x_min = xmin > (float(iframe)-0.5)
				this_x_max = xmax < (float(iframe)+0.5)
				x = [this_x_min, this_x_max, this_x_max, this_x_min, this_x_min]
				yoffset = yscale * (float(ig)+0.5)
				y0 = ymin+yoffset
				;
				y = [y0, y0, y0+dy, y0+dy, y0]
				;
				ind = WHERE( (*r_ptr).color_names[*] eq color, count)
				if(count eq 1) then begin
					color_index = ind[0]
					POLYFILL, x, y, color=color_index
				endif
				;
				loop_end:
			endfor
		not_this_group:
		endfor
		;
		;	repaint the border
		;
		plot, x_plot, y_plot, XRANGE=[xmin, xmax], YRANGE=[ymin, ymax], $
		XSTYLE=1, color=1, background=0, thick=2, YSTYLE=1, LINESTYLE=0, $
		Xtitle = "Image Number", TITLE="LLE RAW1 Mean Counts for night"+night_desig_text, $
		POSITION=(*r_ptr).position,/DEVICE, /NODATA, /NOERASE
	endif
	;
	;	now, paint data
	;
	;if( (*r_ptr).group_flag eq 0) then begin
		d_thick = 1
		s_color = 5
		e_color = 2
	;endif else begin
	;	d_thick = 2
	;	s_color = 1
	;	e_color = 1
	;endelse
	;
	oplot, x_plot, y_plot, color=1, thick=d_thick
	;
	;	don't allow S, E to plot outside window
	;
	s_x = [start_frame, start_frame]
	s_y = [ymax, ymax-0.10*(ymax-ymin)]
	s_y2 = [ymax-0.20*(ymax-ymin),ymin]
	e_x = [end_frame, end_frame]
	e_y = s_y
	e_y2 = s_y2
	if(s_x[0] ge xmin and s_x[0] le xmax) then begin
		oplot, s_x, s_y, color=s_color, thick=3
		oplot, s_x, s_y2, color=s_color, thick=1, LINESTYLE=2
		XYoutS, start_frame, ymax-0.15*(ymax-ymin), 'S', color=s_color
	endif
	if(e_x[0] ge xmin and e_x[0] le xmax) then begin
		oplot, e_x, e_y, color=e_color, thick=3
		oplot, e_x, e_y2, color=e_color, thick=1, LINESTYLE=2
		XYoutS, end_frame, ymax-0.15*(ymax-ymin),'E',color=e_color
	endif
	;
	;	if any jump flags set, mark them
	;
	for iframe = 0, n_pts-1 do begin
		ind = WHERE( x_plot[iframe] eq (*r_ptr).jump_flags[0,0:n_pts-1], count)
		if(count eq 1) then begin
			;
			;	matched image number, check jump flag
			;
			j_flag = (*r_ptr).jump_flags[1,ind[0]]
			if(j_flag ge 1) then begin								; must be flagged
				x_f = x_plot[iframe]
				if((x_f+0.5) ge xmin and (x_f-0.5) le xmax) then begin			; must be in plot window
					xx_f = [x_f, x_f]
					yy_f = [y_plot[iframe], y_plot[iframe]]
					oplot, xx_f, yy_f, color = 12, thick=3, PSYM=4			; pink
				endif
			endif
		endif
	endfor
end
;
;-------------
pro P4_Modify_Plot, Event, CH
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	r_ptr = PTR_struct.PTR_RAW1
	s_ptr = PTR_struct.PTR_summary
	;
	n_pts = (*r_ptr).n_plot
	x_plot = (*r_ptr).x_plot[0:n_pts-1]
	y_plot = (*r_ptr).y_plot[0:n_pts-1]
	;
	xmax = (*r_ptr).xmax
	xmin = (*r_ptr).xmin
	ymax = (*r_ptr).ymax
	ymin = (*r_ptr).ymin
	;
	xmax0 = (*r_ptr).xmax0
	xmin0 = (*r_ptr).xmin0
	;
	end_frame = (*r_ptr).end_frame
	start_frame = (*r_ptr).start_frame
	;
	xx = (*r_ptr).xx
	yy = (*r_ptr).yy
	;
	position = (*r_ptr).position
	;
	;print, "Entered Plot modify with CH = ",CH," and cursor xx,yy = ",xx, yy
	;
	;	first test, be sure the cursor is located within the
	;	plot boundaries
	;
	;if( xx lt position[0] or xx gt position[2]) then xx = position[0]
	;
	;	convert xx device coordinate to data units
	;
	x_data_scale = float(xmax - xmin)
	x_device_scale = float(position[2] - position[0])
	xx_data = round((x_data_scale / x_device_scale) * float(xx - position[0]))
	xx_data = xx_data + xmin
	;
	;print, "xx, xx_data = ",xx, xx_data
	;
	;	do something to the plot, based on the CH character
	;
	if(ch eq 'Z') then begin     ; Zoom command
	    ;
	    ;  expand scale a factor of two about current center
	    ;
	    old_scale = float(xmax - xmin)
	    old_center = (xmax+xmin)/2.
	    ;
	    new_center = xx_data
	    new_scale = (old_scale / 2.)
	    ;
	    (*r_ptr).xmin = xmin0 > (new_center - round(new_scale/2.0))
	    (*r_ptr).xmax = xmax0 < (new_center + round(new_scale/2.))
	    ;
	    goto, replot
	endif
	;
	if(ch eq 'U') then begin   ; Unzoom command
	    ;
	    ;  deexpand scale about new center
	    ;
	    old_scale = float(xmax - xmin)
	    old_center = (xmax+xmin)/2.
	    ;
	    new_center = xx_data
	    new_scale = old_scale * 2.
	    ;
	    (*r_ptr).xmin = xmin0 > (new_center - round(new_scale/2.0))
	    (*r_ptr).xmax = xmax0 < (new_center + round(new_scale/2.))
	    ;
	    goto, replot
	endif
	;
	if(ch eq 'E') then begin   ; reset end_frame
		;
		;print, "On entry to E - start, end frames = ",start_frame, end_frame
		;print, "   xx, xx_data = ",xx, xx_data
		;
	    end_frame = xx_data
	    end_frame_index = n_pts - 1
	end_loop:
	    iend = where(x_plot[*] eq end_frame, count)
	    if(count eq 1) then begin
	    	end_frame_index = iend[0]
	    	(*r_ptr).end_frame = end_frame
	    	MSP_BDP_print_text, Event, "End-Frame No = "+STRING(format='(I)',end_frame)
	    endif else begin
	 	    MSP_BDP_print_text, Event, "WARNING - No Image ("+STRING(format='(I)',end_frame)+") Trying previous image"
	        end_frame_index = n_pts - 1
	        end_frame = (end_frame-1) > 0
	        if(end_frame eq 0) then begin
	            MSP_BDP_print_text, Event,"ERROR - Could not find ending frame number. Please reselect"
	        endif else begin
	            goto, end_loop
	        endelse
	    endelse
	    MSP_BDP_print_text, Event, "End-Frame reset to "+STRING(format='(I)',end_frame)
	    goto, replot
	endif
	;
	if(ch eq 'S') then begin   ; reset start_frame
	    start_frame = xx_data
	    start_frame_index = 0
	 start_loop:
	 	istart = where(x_plot[*] eq start_frame,count)
	    if(count eq 1) then begin
	    	start_frame_index = istart[0]
	    	(*r_ptr).start_frame = start_frame
	    	MSP_BDP_print_text, Event, "Start-Frame No = "+STRING(format='(I)',start_frame)
	    endif else begin
	        MSP_BDP_print_text,Event, "WARNING - No Image ("+STRING(format='(I)',start_frame)+") Trying next frame"
	        start_frame_index=0
	        start_frame = (start_frame +1) < x_plot[n_pts-1]
	        if(start_frame eq x_plot[n_pts-1]) then begin
	            MSP_BDP_print_text, event,"ERROR - Could not find starting frame number. Please reselect"
	        endif else begin
	            goto, start_loop
	        endelse
	    endelse
	    MSP_BDP_print_text, Event, "Start-Frame reset to "+STRING(format='(I)',start_frame)
	    goto, replot
	endif
	;
	if(ch eq 'P') then begin 		; plot from start to end command
		(*r_ptr).xmin = (*r_ptr).start_frame > (*r_ptr).xmin0
		(*r_ptr).xmax = (*r_ptr).end_frame < (*r_ptr).xmax0
		goto, replot
	endif
	;
	if(ch eq 'G') then begin		; toggle group plotting
		(*r_ptr).group_flag = 1 - (*r_ptr).group_flag
		if( (*r_ptr).group_flag lt 0 or (*r_ptr).group_flag gt 1) then (*r_ptr).group_flag = 0
		goto, replot
	endif
	;
	if(ch eq 'J') then begin		; select one image for jump flag toggling
		ind = WHERE(xx_data eq x_plot[*],count)
		if(count eq 1) then begin
			image_no = x_plot[ind[0]]
			;
			jnd = WHERE( (*r_ptr).jump_flags[0,*] eq image_no,jcount)
			if(jcount eq 1) then begin
				(*r_ptr).jump_flags[1,jnd[0]] = 1 - (*r_ptr).jump_flags[1,jnd[0]]
				if((*r_ptr).jump_flags[1,jnd[0]] lt 0 or (*r_ptr).jump_flags[1,jnd[0]] gt 1) then $
										(*r_ptr).jump_flags[1,jnd[0]] = 0
			endif
			;
			MSP_BDP_print_text, Event, "Marked/Added RAW1 Jump for Image "+STRING(format='(I)',image_no)
		endif
		goto, replot
	end
	;
	if(ch eq 'C') then begin		; clear all jumps
		(*r_ptr).jump_flags[1,*] = 0
		MSP_BDP_print_text, Event, "Cleared all Jump flags"
		goto, replot
	endif
	;
	if(ch eq 'F') then begin		; find all jumps
		;
		;	first, check that start and end limits are correctly ordered
		;
		if( (*r_ptr).start_frame gt (*r_ptr).end_frame) then begin
			MSP_BDP_print_text, Event, "ERROR - start and end misordered - try again."
			goto, start_loop
		endif else begin
			P4_Find_Jumps, Event
			goto, replot
		endelse
	endif
	;
	;	reissue plot command
	;
replot:
	;
	P4_RAW1_Plot, Event
	;
	P4MP_end:
end
;
;-------------
pro P4_Find_Jumps, Event
	;
	;   have selected image number range to search for jumps
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	r_ptr = PTR_struct.PTR_RAW1
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	;
	n_pts = (*r_ptr).n_plot
	x_plot = (*r_ptr).x_plot[0:n_pts-1]
	y_plot = (*r_ptr).y_plot[0:n_pts-1]
	;
	xmax = (*r_ptr).xmax
	xmin = (*r_ptr).xmin
	ymax = (*r_ptr).ymax
	ymin = (*r_ptr).ymin
	;
	xmax0 = (*r_ptr).xmax0
	xmin0 = (*r_ptr).xmin0
	;
	end_frame = (*r_ptr).end_frame
	start_frame = (*r_ptr).start_frame
	;
	max_image = (*n_ptr).max_image
	;
	;   next, for each of the 8 data vectors, compute median_filtered_means and dispersions
	;			across just this start-end range
	;
	;raw1_detect = {image_no: 0L, mean_even: fltarr(4), s_mean_even: fltarr(4),$
	;				mean_odd: fltarr(4), s_mean_odd: fltarr(4)}
	;
	n_even_odd = 2
	n_quad = 4
	;
	mean_values = dblarr(n_even_odd,n_quad)
	dispersions = dblarr(n_even_odd,n_quad)
	;
	deviation_RAW1 = dblarr(2, 4, n_pts)			; local table of deviations (not the full one)
	deviation_flag = intarr(2,4,n_pts)
	;
	n_jumps = 0L
	;
	WIDGET_CONTROL, /HOURGLASS
	;
	for i_even_odd = 0, 1 do begin
	    for i_quad = 0, 3 do begin
	    	;
	    	;	get data for each image in the indicated range
	    	;
	    	vector = dblarr(max_image)
	    	img_save = lonarr(max_image)
	    	;
	    	ivec = 0L
	    	;
	    	for iframe = start_frame, end_frame do begin
	    		;
	    		;	match to data
	    		;
	    		ind = WHERE( (*r_ptr).detect[*].image_no eq iframe, count)	; find which image in the examination matches
	    		if(count eq 1) then begin
	    			img_save[ivec] = iframe
	    			if(i_even_odd eq 0) then begin
	    				vector[ivec] = (*r_ptr).detect[ind[0]].mean_even[i_quad]
	    				ivec++
	    			endif else begin
	    				vector[ivec] = (*r_ptr).detect[ind[0]].mean_odd[i_quad]
	    				ivec++
	    			endelse
	    		endif
	    	endfor
	    	vector = vector[0:ivec-1]			; the values
	    	img_save = img_save[0:ivec-1]		; their image numbers
	    	;
	       	zz = median_filtered_mean(vector)	; get mean, standard deviation for this quad, o/e filtered set
	        ;	load
	        mean_values[i_even_odd,i_quad] = zz[0]
	        dispersions[i_even_odd,i_quad] = zz[1]
	        ;
	        ; now, flag each entry that is more than 3 sigma away from mean
	        ;
	        for ii = 0, ivec-1 do begin
	         ;
	         if(abs(vector[ii] - zz[0])/zz[1] gt 3.) then begin
	          	deviation_flag[i_even_odd,i_quad,ii] = 1
	         endif else begin
	          	deviation_flag[i_even_odd,i_quad,ii] = 0
	         endelse
	         deviation_RAW1[i_even_odd,i_quad,ii] = vector[ii] - zz[0]
	       endfor
	    endfor
	endfor
	;
	;   all data processed, now evaluate across all quads, o/e rows for a given image
	;   if > half of the 8 entries have deviations, flag this image as deviant
	;   else, flag it as non-deviant
	;
	for ii = 0, ivec-1 do begin     ; for each frame
	    nbad = 0
	    for i_even_odd = 0, 1 do begin
	       for i_quad = 0, 3 do begin
	         if(deviation_flag[i_even_odd,i_quad,ii]) then nbad++
	       endfor
	    endfor
    	;
    	;	set flag
    	;
    	image_no = img_save[ii]
    	ind = WHERE( (*r_ptr).detect[*].image_no eq image_no, count)
    	;
    	if(count eq 1) then begin
    		i_match = ind[0]
    		;
    		if(nbad ge 4) then begin
	    		(*r_ptr).jump_flags[1,i_match] = 1
	    		(*r_ptr).jump_flags[0,i_match] = image_no
	    		n_jumps++
	    		;
	    		;	load deviation structure as well
	    		;
;	    		(*r_ptr).deviation_RAW1[*,*,i_match] = deviation_RAW1[*,*,ii]
	    	endif else begin
	    		(*r_ptr).jump_flags[1,i_match] = 0
	    		(*r_ptr).jump_flags[0,i_match] = image_no
;	    		(*r_ptr).deviation_RAW1[*,*,ind[0]] = 0.0d0
	    	endelse
	    	;
    		;	load deviation structure as well
    		;
    		(*r_ptr).deviation_RAW1[*,*,i_match] = deviation_RAW1[*,*,ii]
    	endif
	endfor
	;
	;	replot
	;
	P4_RAW1_Plot, Event
	;
	if(n_jumps gt 0) then begin
		MSP_BDP_print_text, Event, "Found "+STRING(format='(I)',n_jumps)+" RAW1 Jumps"
	endif else begin
		MSP_BDP_print_text, Event, "Did not find any RAW1 Jumps between Start and End Image numbers"
	endelse
end
;
;-------------
pro P4_Done, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	n_ptr = PTR_struct.PTR_night
	r_ptr = PTR_struct.PTR_RAW1
	;
	n_images = (*n_ptr).num_images
	ind = WHERE( (*r_ptr).jump_flags[1,0:n_images] eq 1,count)
	;
	MSP_BDP_print_text, Event,"Ready to try to fix "+STRING(format='(I)',count)+' images with RAW1 jumps'
	;
	;	move on to jump corrector
	;	turn on remaining button
	;
	P4_button_sensitize, Event, 12, 1
	;
	MSP_BDP_print_text, Event, "Jump Detection completed."
end
;
;-------------
pro P4_Correct_Jumps, Event
	;
	MSP_BDP_print_text, Event, "Repair of flagged RAW1 jumps requested"
	;
	;	for each jump needed, fix the raw1 image and bump the image_root_index for the image
	;	then output the new images in the new directory
	;
	;	but, respect deleted pixels in RAW1 images
	;
	;	also, scale back by coadds, if not 1
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	n_ptr = PTR_struct.PTR_night
	;
	;	for each image with a jump flag set...
	;
	n_pts = (*r_ptr).n_plot					; number of image frames in examination
	n_night_list = (*n_ptr).num_images		; number of images listed on night's image list
	;
	ind = WHERE( (*r_ptr).jump_flags[1,0:n_pts-1] eq 1, count)
	if(count ge 1) then begin
		n_jumps = count
	endif else begin
		MSP_BDP_print_text, Event, "ERROR - No RAW1 Jumps to Repair.. Skipping"
		goto, P4_CJ_end
	endelse
	;
	n_LED = N_ELEMENTS( (*s_ptr).P4_LED_colors)
	(*s_ptr).P4_LED_colors[*] = "yellow"
	LED_scale = float(n_jumps) / float(n_LED)
	if(LED_scale lt 1.0) then LED_scale = 1.0
	if(n_jumps lt n_LED) then (*s_ptr).P4_LED_colors[n_jumps:n_LED-1] = 'black'
	WCT_LED_stack, (*s_ptr).P4_LED_wid[1], (*s_ptr).P4_LED_colors
	;
	last_LED = 0
	;
	raw_bad_pix_value = UINT(0)
	;
	WIDGET_CONTROL, /HOURGLASS
	;
	i_jumps = 0L
	;
	for iframe = 0, n_pts-1 do begin
		;
		this_LED = ROUND(float(i_jumps) / LED_scale)
		this_LED = 0 > ((n_LED-1) < this_LED)
		(*s_ptr).P4_LED_colors[this_LED] = "orange"
		WCT_LED_stack, (*s_ptr).P4_LED_wid[1], (*s_ptr).P4_LED_colors
		;
		if( (*r_ptr).jump_flags[1,iframe] eq 1) then begin
			;
			this_LED = ROUND(float(i_jumps) / LED_scale)
			this_LED = 0 > ((n_LED-1) < this_LED)
			(*s_ptr).P4_LED_colors[this_LED] = "blue"
			WCT_LED_stack, (*s_ptr).P4_LED_wid[1], (*s_ptr).P4_LED_colors
			;
			;	jump found, get image no
			;
			image_no = (*r_ptr).jump_flags[0,iframe]
			;
			;	create input and output pathnames via lookup ------------------------------
			;
			;	1. match image no to night list
			;
			ind = WHERE( (*n_ptr).image_list[0:n_night_list-1] eq image_no, count)
			if(count ne 1) then begin
				MSP_BDP_print_text, Event, "ERROR - did not match image number in P4 Correct Jumps"
				goto, image_loop_end
			endif
			;
			i_match = ind[0]
			;
			;	save coadd number for this image
			;
			this_coadds = (*n_ptr).image_coadds[i_match]
			;
			;	2. get pathnames for input from index for this image
			;
			dir_index = (*n_ptr).image_root_index[i_match]
			;
			;	and the directories
			;
			input_RAW1_dir = 	(*n_ptr).paths_raw1_root[dir_index]
			input_CDS_dir = 	(*n_ptr).paths_cds_root[dir_index]
			;
			if(dir_index eq 0) then begin
				base_dir = (*s_ptr).root_raw
			endif else begin
				base_dir = (*s_ptr).root_reduced
			endelse
			;
			input_RAW1_dir = base_dir + input_RAW1_dir
			input_CDS_dir = base_dir + input_CDS_dir
			;
			image_no_text = STRTRIM( STRING(format='(I5.3)',image_no),2)
			input_RAW1_pathname = fix_path( FILEPATH('raw1_'+image_no_text, ROOT_DIR=input_RAW1_dir) )
			night_desig_text = STRING(format='(I8.8)',(*n_ptr).night_desig)
			input_CDS_pathname = fix_path( FILEPATH(night_desig_text+"."+image_no_text+".fits", $
								ROOT_DIR=input_CDS_dir) )
			;
			;	test that both exist
			;
			test_RAW1 = FILE_INFO(input_RAW1_pathname)
			test_CDS = FILE_INFO(input_CDS_pathname)
			;
			if( NOT(test_RAW1.exists) or NOT(test_CDS.exists) ) then begin
				MSP_BDP_print_text, Event, "ERROR - either input RAW1 or CDS image does not exist. skipping"
				goto, image_loop_end
			endif
			;
			;	create the new output image pathnames and directories
			;
			output_RAW1_name = FILE_BASENAME(input_RAW1_pathname)
			output_CDS_name = FILE_BASENAME(input_CDS_pathname)
			;
			;	create and test directories
			;
			output_dir = (*s_ptr).root_reduced + '4_RAW1_Jump_Corrected' + PATH_SEP()
			test_dir = FILE_INFO(output_dir)
			if( NOT(test_dir.exists) or NOT(test_dir.directory)) then begin
				FILE_MKDIR, output_dir
			endif
			night_dir = output_dir + night_desig_text + PATH_SEP()
			test_night = FILE_INFO(night_dir)
			if( NOT(test_night.exists) or NOT(test_night.directory)) then begin
				FILE_MKDIR, night_dir
			endif
			;
			output_RAW1_dir = night_dir + 'RAW1_RAW' + PATH_SEP()
			test_RAW1 = FILE_INFO(output_RAW1_dir)
			if( NOT(test_RAW1.exists) or NOT(test_RAW1.directory)) then begin
				FILE_MKDIR, output_RAW1_dir
			endif
			(*n_ptr).paths_raw1_root[2] = '4_RAW1_Jump_Corrected' + PATH_SEP() + night_desig_text + PATH_SEP() + 'RAW1_RAW' + PATH_SEP()
			;
			output_RAW1_pathname = fix_path( FILEPATH(output_RAW1_name, ROOT_DIR=output_RAW1_dir) )
			;
			output_CDS_dir = night_dir + 'CDS_RAW' + PATH_SEP()
			test_CDS = FILE_INFO(output_CDS_dir)
			if( NOT(test_CDS.exists) or NOT(test_CDS.directory)) then begin
				FILE_MKDIR, output_CDS_dir
			endif
			(*n_ptr).paths_cds_root[2] = '4_RAW1_Jump_Corrected' + PATH_SEP() + night_desig_text + PATH_SEP() + 'CDS_RAW' + PATH_SEP()
			;
			output_CDS_pathname = fix_path( FILEPATH(output_CDS_name, ROOT_DIR=output_CDS_dir) )
			;
			;	reset index pointer for this image
			;
			(*n_ptr).image_root_index[i_match] = 2L
			;
			; --------------------------------------------------------------------------------
			;
			;	read the input images
			;
			CDS_image = readfits(input_CDS_pathname, CDS_header, /SILENT)
			RAW1_image = read_raw1(input_RAW1_pathname)
			;
			if(this_coadds ne 1) then begin		; reduce by coadds
				RAW1_image /= this_coadds
			endif
			;
			r_size = SIZE(RAW1_image)
			;
			;print, "RAW1 image size = (RxC) = ",r_size[1],r_size[2]
			;
			;print, "Deviation array for image_no = ",(*n_ptr).image_list[i_match]
			;
			;for ioe = 0, 1 do print, "odd/even = ",ioe," ",(*r_ptr).deviation_RAW1[ioe,0,iframe],$
			;    (*r_ptr).deviation_RAW1[ioe,1,iframe],$
			;    (*r_ptr).deviation_RAW1[ioe,2,iframe],$
			;    (*r_ptr).deviation_RAW1[ioe,3,iframe]
			;
			;	and deal with them.
			;
			;	-- for modern (post 2005) data, just copy CDS image, but put in a note
			;		to the FITS header
			;
			FXADDPAR, CDS_header, "HISTORY", "Repaired RAW1 jumps in this image"
			writefits, output_CDS_pathname, CDS_image, CDS_header
			;
			this_LED = ROUND(float(i_jumps) / LED_scale)
			this_LED = 0 > ((n_LED-1) < this_LED)
			(*s_ptr).P4_LED_colors[this_LED] = "violet"
			WCT_LED_stack, (*s_ptr).P4_LED_wid[1], (*s_ptr).P4_LED_colors
	        ;
	        ;	--- for RAW1 image, must remove flagged deviations from odd/even rows by quadrant
	        ;
	        ;   		-- NOTE!! full frame only for now
            ;
            for i_left_right = 0, 1 do begin
                icol0 = i_left_right * 512
                icol1 = icol0 + 511
                for i_bottom_top = 0, 1 do begin
                   irow0 = i_bottom_top * 512
                   irow1 = irow0 + 511 + 2 * i_bottom_top     ; top is 514 rows tall
                   ;
                   iquad = i_left_right + 2 * i_bottom_top
                   ;
                   ; for even, then odd rows
                   ;
                   for i_even_odd = 0, 1 do begin
                     ;
                     irow0 = irow0 + i_even_odd
                     irow1 = irow1 + i_even_odd
                     ;
                     ;    correct all the relevant RAW1 pixels
                     ;
                     for irow = irow0, irow1, 2 do begin
                      for icol = icol0, icol1 do begin
                          if(RAW1_image[icol,irow] ne raw_bad_pix_value) then begin
                             RAW1_image[icol,irow] = RAW1_image[icol,irow] - UINT((*r_ptr).deviation_RAW1[i_even_odd, iquad, iframe])
                          endif else begin
                             RAW1_image[icol,irow] = raw_bad_pix_value
                          endelse
                          ;
                      endfor
                     endfor
                   endfor
	            endfor
	        endfor
            ;
            ;   save corrected images
            ;
            ;	if coadded, scale back to coadd values
            ;
            if(this_coadds ne 1) then begin
            	RAW1_image *= this_coadds
            endif
            ;
            ier = write_raw1(output_RAW1_pathname, RAW1_image)
            ;
            this_LED = ROUND(float(i_jumps) / LED_scale)
			this_LED = 0 > ((n_LED-1) < this_LED)
			(*s_ptr).P4_LED_colors[this_LED] = "green"
			WCT_LED_stack, (*s_ptr).P4_LED_wid[1], (*s_ptr).P4_LED_colors
			;
			i_jumps++
	    endif
	    ;
	    image_loop_end:
	endfor
	;
	MSP_BDP_print_text, Event, "All RAW1 Jumps Corrected."
	;
	P4_CJ_END:
	;
	;	save the night summary to disk as an IDL sav file
	;
	night_desig = (*n_ptr).night_desig
	;
	MSP_BDP_Create_Night_File_Path, Event, night_desig, night_file_path
    ;
	MSP_BDP_Save_Night_File, Event, night_file_path
	MSP_BDP_print_text, Event, "Saved Night Save file for night "+STRING(format='(I8.8)',night_desig)
	;
	;	save night summary
	;
	P4_Save_Night_Summary, Event
	;
	;	and turn off all buttons and clear the draw window and write "Done !"
	;
	for i = 0, 12 do P4_button_sensitize, Event, i, 0
	P4_LED_sensitize, Event, 0, 0
	P4_LED_sensitize, Event, 1, 0
	;
	WSET, (*s_ptr).P4_draw_window_id
	ERASE
	;
	device, decomposed=0
	color_red = REFORM((*r_ptr).color_rgb[0,*])
	color_green = REFORM((*r_ptr).color_rgb[1,*])
	color_blue = REFORM((*r_ptr).color_rgb[2,*])
	;
	tvlct, color_red, color_green, color_blue, 0
	;
	window_id = (*s_ptr).P4_draw_window_id
	;
	WSET, window_id
	;
	x = [0., 1.]
	y = [0., 1.]
	;
	plot, x, y, color=0, xrange=[0.,1.], yrange=[0.,1.], xstyle=1, ystyle=1, $
				background=0, thick=1, LINESTYLE=0, $
				POSITION=(*r_ptr).position,/DEVICE,/NODATA
	;
	XYoutS, 0.5, 0.5, "Done !", ALIGNMENT=0.5, color=2, charthick=2, charsize=4.0
	;
end
;
;-----------------------------------------------------------------
pro P4_Save_Night_Summary, Event
	;
	;	saves an updated night summary
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	;
	;	how may RAW1 jumps were found and fixed?
	;
	n_pts = (*r_ptr).n_plot
	ind = WHERE( (*r_ptr).jump_flags[1,0:n_pts-1] eq 1, count)
	n_jumps = count
	;
	this_night = (*n_ptr).night_desig
	;
	;	which one in the summary?
	;
	num_nights = (*s_ptr).num_nights
	ind = WHERE(this_night eq (*s_ptr).night_date[0:num_nights-1], count)
	if(count ne 0) then night_index = ind[0] else begin
		MSP_BDP_print_text, Event, "Error - No matching night in P4..."
		goto, P4SNS_end
	endelse
	;
	(*s_ptr).night_who[night_index] = (*s_ptr).who
	if(n_jumps eq 0) then begin
		(*s_ptr).night_step[night_index] = 4
		(*s_ptr).night_comment[night_index] = "No RAW1 Jumps"
	endif else begin
		(*s_ptr).night_step[night_index] = 4
		(*s_ptr).night_comment[night_index] = "RAW1 Jumps Fixed"
	endelse
	;
	P1_Write_Summary_File, Event
	;
	P4SNS_end:
end
