pro MSP_BDP_Read_Ave_RAW1_Headers, Event, summary_struct
	;
	;	read the Ave RAW1 directory to get the image list,
	;	then read each header to get the night and UT
	;	for each image into a summary structure
	;
	MSP_BDP_print_text, Event, "Fetch Ave RAW1 List Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	f_ptr = PTR_struct.PTR_FITS
	;b_ptr = PTR_struct.PTR_BPM
	a_ptr = PTR_struct.PTR_Ave_RAW1
	;
	;	first, build the directory path to search
	;
	root_reduced = (*s_ptr).root_reduced
	Ave_RAW1_directory = (*a_ptr).output_directory		;
	if(Ave_RAW1_directory eq '') then Ave_RAW1_directory = '7_Ave_RAW1_Images'
	;
	Ave_RAW1_dir_path = root_reduced + Ave_RAW1_directory + PATH_SEP()
	;
	;print, "Ave_RAW1_path = ",Ave_RAW1_dir_path
	;
	;	get list of images in that directory
	;
	test_dir = FILE_INFO(Ave_RAW1_dir_path)
	if( NOT(test_dir.exists) or NOT(test_dir.directory) ) then begin
		MSP_BDP_print_text, Event, "ERROR - Ave RAW1 directory path incorrect"
		goto, MBPBH_end
	endif
	;
	Ave_RAW1_list = FILE_SEARCH(Ave_RAW1_dir_path+'*.fits')
	;
	;print, "BPM_list = ",BPM_list
	;
	n_ar = N_ELEMENTS(Ave_RAW1_list)
	;
	MSP_BDP_print_text, Event, "Found "+STRING(format='(I)',n_ar)+" Ave RAW1 images."
	;
	;	open each and build the date, UT, path list
	;
	(*a_ptr).n_groups = n_ar
	;
	for iar = 0, n_ar-1 do begin
		ar_path = Ave_RAW1_list[iar]
		(*a_ptr).group[iar].image_CDS_pathname[0] = ar_path
		;
		;	read the Ave RAW1 image
		;
		header_ar = headfits(ar_path, /silent)
		;
		;	load buffers
		;
		n_header_lines = N_ELEMENTS(header_ar)
		(*s_ptr).raw_header[0:n_header_lines-1] = header_ar
		(*s_ptr).raw_header[n_header_lines:N_ELEMENTS((*s_ptr).raw_header)-1] = ''
		;
		;	extract key values
		;
		MSP_BDP_Parse_FITS_Header, Event
		;
		(*a_ptr).group[iar].coadds = (*f_ptr).numcoadds
		;
		(*a_ptr).group[iar].ut = (*f_ptr).ut
		raw_filename = (*f_ptr).raw_filename
		night_desig = 0L
		reads, raw_filename, format='(14x,i8)',night_desig
		(*a_ptr).group[iar].night_desig = night_desig
		;
		;	calc JD of ave raw1
		;
		ar_desig = (*a_ptr).group[iar].night_desig
		year = ar_desig / 10000L
		month = (ar_desig - 10000L * year) / 100L
		day = ar_desig - 10000L * year - 100L * month
		;
		ar_UT = (*a_ptr).group[iar].ut
		hour = FLOOR(ar_UT)
		minute = FLOOR((ar_UT - double(hour))*60.0d0)
		second = 3600.0d0 * (ar_UT - double(hour) - double(minute)/60.0d0)
		;
		ar_jd = JULDAY(month, day, year, hour, minute, ROUND(second))
		;
		(*a_ptr).group[iar].jd = ar_jd
		;
		;print, "AR No. ",iar," Night, UT = ",ar_desig, (*a_ptr).group[iar].ut, " path = ",ar_path, $
		;	" JD = ",ar_jd
	endfor
	;
	MBPBH_end:
end