pro MSP_BDP_Ski_Jump_Detector, image, ski_jump_code
;
;	tests for the presence of ski jumps at top and bottom of
;	Mimir images, using an N sigma criterion
;	;
;	20200105 - reevaluated to give two center reference regions
;	            and thus avoid the top/bottom DC offset problem
;	            feeding into the rms for the sigma test
;
;	criterion
;
criterion = 5.0
;
; debug
; 
idebug = 0
;
image_size = SIZE(image)
nc = image_size[1]
nr = image_size[2]
;
;	select first half of image
;
ncx = ROUND(nc/4)
;
;	and column strip to examine
;
ncx1 = 0 > (ncx-5)
ncx2 = nc < (ncx+5)
;
dncx = abs(ncx2 - ncx1) + 1
;
;	center reference regions
;
nry_top = ROUND(nr/2) + 20
nry_bot = ROUND(nr/2) - 20
;
nr_ref_1_top = 0 > (nry_top-6)
nr_ref_2_top = nr < (nry_top+5)
nr_ref_1_bot = 0 > (nry_bot-6)
nr_ref_2_bot = nr < (nry_bot+5)
;
dnr_ref_top = abs(nr_ref_2_top - nr_ref_1_top) + 1
dnr_ref_2_top = dnr_ref_top / 2
dnr_ref_bot = abs(nr_ref_2_bot - nr_ref_1_bot) + 1
dnr_ref_2_bot = dnr_ref_bot / 2
;
;	bottom ski jump test region
;
nr_bot_1 = 0
nr_bot_2 = 3
;
dnr_bot = abs(nr_bot_2 - nr_bot_1) + 1
;
;	top ski jump test region
;
nr_top_2 = nr-3
nr_top_1 = nr-6
;
dnr_top = abs(nr_top_2 - nr_top_1) + 1
;
;	compute median, rms for center reference regions, on
;	and even-odd row basis to account for O/E readout
;
even_top = FLOOR(nr_ref_1_top/2)*2
odd_top = even_top + 1
even_bot = FLOOR(nr_ref_1_bot/2)*2
odd_bot = even_bot + 1
;
center_even_top = fltarr(dncx,dnr_ref_2_top)
center_odd_top = fltarr(dncx,dnr_ref_2_top)
center_even_bot = fltarr(dncx,dnr_ref_2_bot)
center_odd_bot = fltarr(dncx,dnr_ref_2_bot)
;
for irow = 0, dnr_ref_2_top-1 do begin
	jrow = even_top + 2*irow
	center_even_top[*,irow] = image[ncx1:ncx2,jrow]
endfor
for irow = 0, dnr_ref_2_top-1 do begin
	jrow = odd_top + 2*irow
	center_odd_top[*,irow] = image[ncx1:ncx2,jrow]
endfor
;
for irow = 0, dnr_ref_2_bot-1 do begin
  jrow = even_bot + 2*irow
  center_even_bot[*,irow] = image[ncx1:ncx2,jrow]
endfor
for irow = 0, dnr_ref_2_bot-1 do begin
  jrow = odd_bot + 2*irow
  center_odd_bot[*,irow] = image[ncx1:ncx2,jrow]
endfor
;
center_e_top = REFORM(center_even_top[*,*], dncx*dnr_ref_2_top )
center_o_top = REFORM(center_odd_top[*,*], dncx*dnr_ref_2_top)
;
center_e_bot = REFORM(center_even_bot[*,*], dncx*dnr_ref_2_bot)
center_o_bot = REFORM(center_odd_bot[*,*], dncx*dnr_ref_2_bot)
;
if(idebug eq 1) then help, center_e_top,center_o_top,center_e_bot,center_o_bot
;
ref_e_top = median_filtered_mean(center_e_top)
ref_o_top = median_filtered_mean(center_o_top)
;
ref_e_bot = median_filtered_mean(center_e_bot)
ref_o_bot = median_filtered_mean(center_o_bot)
;
if(idebug eq 1) then print, "top center even, odd mean, rms = ",ref_e_top,ref_o_top
if(idebug eq 1) then print, "bot center even, odd mean, rms = ",ref_e_bot,ref_o_bot
;
;	for top, bottom, do MFM along rows first
;
bot_vec = fltarr(dnr_bot)
top_vec = fltarr(dnr_top)
;
for nr = 0, dnr_bot-1 do begin
	nrr = nr + nr_bot_1
	zz = median_filtered_mean(REFORM(image[ncx1:ncx2,nrr:nrr]))
	bot_vec[nr] = zz[0]
endfor
for nr = 0, dnr_top-1 do begin
	nrr = nr + nr_top_1
	zz = median_filtered_mean(REFORM(image[ncx1:ncx2,nrr:nrr]))
	top_vec[nr] = zz[0]
endfor
;
;	compute average top, bottom
;
bottom = MOMENT(bot_vec)
top = MOMENT(top_vec)
;
ski_jump = intarr(2)
;
t1 = (bottom[0] - ref_e_bot[0])/ref_e_bot[1]
t2 = (bottom[0] - ref_o_bot[0])/ref_o_bot[1]
t3 = (top[0] - ref_e_top[0])/ref_o_top[1]
t4 = (top[0] - ref_o_top[0])/ref_o_top[1]
;
if(t1 gt criterion or t2 gt criterion) then ski_jump[0] = 1 else ski_jump[0] = 0
if(t3 gt criterion or t4 gt criterion) then ski_jump[1] = 1 else ski_jump[1] = 0
;
if(idebug eq 1) then print, "ski jump array = ",ski_jump
if(idebug eq 1) then print, "bottom moment array = ",bottom
if(idebug eq 1) then print, "top moment array = ",top
if(idebug eq 1) then print, "t1, t2, t3, t4 = ",t1,t2,t3,t4
;
ski_jump_code = ski_jump[0] + 10 * ski_jump[1]
;
end