pro P12_fix_header_HWP_angle, Event, image, n_45
;
;	advances the HWP angle info in the image's FITS header
;	by n_45 45 degree steps - ie, if n_45 = 3, then the
;	FITS header variables are advanced by 3 * 22.5 degrees
;
;	DPC	20111218
;
WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
n_ptr = PTR_struct.PTR_night
s_ptr = PTR_struct.PTR_summary
f_ptr = PTR_struct.PTR_FITS
bdf_ptr = PTR_struct.PTR_BDF
rt_ptr = PTR_struct.PTR_RTime
;
;	read the image
;
image_data = readfits(image,this_header,/silent)
;
;	check for the presence of the new header variable that calls out the Zero Hop fix
;		already done
;
this_zero_hop = sxpar(this_header, 'MB_PFIX', COUNT=count)
if(count gt 0) then begin
	MSP_BDP_print_text, Event, 'ERROR - trying to Zero Hop fix an already fixed image'
	MSP_BDP_print_text, Event, '      - image = '+FILE_BASENAME(input_pathnames[ind_fid[i_fid]])
	MSP_BDP_print_text, Event, '      - skipping this image'
	goto, no_write
endif
;
;	and change the header by adding 22.5 deg (33 HWP motor steps) increments to its count
;
n_add = 33 * ROUND(n_45)	; steps
;
this_comment = sxpar(this_header, 'COMMENT')
n_comment = N_ELEMENTS(this_comment)
;
;	find the BD1 line, if it is there
;
bd1_flag = 0
;
for iline = 0, n_comment-1 do begin
	ipos = STRPOS( STRUPCASE( this_comment[iline] ), 'BD1')
	if(ipos ne -1) then begin
		comment1 = STRUPCASE( this_comment[iline] )
		s0 = STRPOS( comment1, "GOTO")
		s1 = STRPOS( comment1, '.')
		s3 = STRPOS( comment1, 'HOME')
		;
		istep = 0L
		if(s0 ge 0) then begin
			if(s1 ge 0) then begin
				if(s3 ge 0) then begin
					istep = 0
				endif else begin
					hwp_string = STRMID( comment1, s0+5, (s1-s0-5))
					reads, hwp_string, format='(i)', istep
				endelse
			endif else begin
				;
				;	no dots... so, parse with spaces
				;
				len = STRLEN(comment1) - s0
				remainder = STRMID(comment1, s0, len-1)
				parts = STRSPLIT(remainder, ' ',/REGEX,/EXTRACT)
				len1 = STRLEN(parts[1])			; "GOTO" is parts[0]
				if(len1 ne 0) then begin
					nstep = STRTRIM(parts[1],2)
					reads, nstep, format='(i)', istep
				endif else istep = -99
			endelse
		endif else begin
			if(s3 ge 0) then begin
				istep = 0
			endif else begin
				istep = -99.9
			endelse
		endelse
		;
		;	add n_45 * 33 steps (22.5 deg)					67 steps (45 deg)
		;
		if(istep ne -99) then istep = istep + n_add
		;
		;	check against full loop
		;
		if(istep gt 533) then istep = istep - 533
		;
		;	if in range of 525 to 8, set = home
		;
		if(istep gt 525 or (istep ge 0 and istep le 8)) then istep = 0
		;
		;	remove the comments from this FITS header
		;
		sxdelpar, this_header, 'COMMENT'
		;
		;	and write a new set
		;
		if(istep ne 0 and istep ne -99) then begin
			this_string = STRTRIM(STRING(istep),2)
			sxaddpar, this_header, 'COMMENT', 'BD1 goto '+this_string+'... done'
			if(N_ELEMENTS(this_comment) ge (iline+2)) then begin
				sxaddpar, this_header, 'COMMENT', this_comment[iline+1]
			endif
		endif else begin
			;
			if(istep eq 0) then begin
				sxaddpar, this_header, 'COMMENT', 'BD1 goto home..off'
				if(N_ELEMENTS(this_comment) ge (iline+2)) then begin
					sxaddpar, this_header, 'COMMENT', this_comment[iline+1]
				endif
			endif else begin
				;
				;	angle = -99 here - call out problem
				;
				MSP_BDP_print_text, Event, 'ERROR - HWP angle = -99 for image '+FILE_BASENAME(input_pathnames[ind_fid[i_fid]])
				goto, no_write
			endelse
		endelse
		;
		;	correction to older data made. move on to newer data
		;
		goto, update_header
	endif
endfor
;
;	no BD1 variables, fetch new ones.
;
n_header_lines = N_ELEMENTS(this_header)
(*s_ptr).raw_header[0:n_header_lines-1] = this_header
MSP_BDP_Parse_FITS_Header, Event
MSP_BDP_Parse_HWP_Angle, Event, HWP_angle
;
;	add offset to HWP_angle
;
HWP_angle += n_45 * 22.5
;
if(HWP_angle gt 360.0) then HWP_angle -= 360.0
if(HWP_angle lt 0.0) then HWP_angle += 360.0
istep = ROUND(HWP_angle / 0.675)
;
sxaddpar, this_header, 'HWP_ANG', HWP_angle, 'P12 - had '+STRING(n_45*22.5,format='(F6.1)')+' added to it'
sxaddpar, this_header, 'HWP', istep,'P12 - calculated HWP step'
if(HWP_angle lt 3.0 or HWP_angle gt 357.0) then begin
	sxaddpar, this_header, 'H_HOME', 'T', 'P12 - reset'
endif else begin
	sxaddpar, this_header, 'H_HOME', 'F', 'P12 - updated'
endelse
;
update_header:
;
;	add some comments and a header variable
;
sxaddpar, this_header, 'MB_PFIX', 1, 'This image had its HWP angle changed in S12 BDP'
sxaddpar, this_header, 'HISTORY', 'HWP angle modified '+SYSTIME()+' due to Zero Hop problem
;
;	and change the HWP angle in an MB_ variable
;
sxdelpar, this_header, 'MB_HWP'
;
new_ang = normal_angle(float(istep) * 0.675)
sxaddpar, this_header, 'MB_HWP', new_ang,'MSP-BDP HWP Angle in degrees'
;
;	and update the night summary
;
n_night_images = (*n_ptr).num_images
ind_this = WHERE((*n_ptr).image_name[0:n_night_images-1] eq FILE_BASENAME(image,'_LC.fits',/FOLD_CASE),count)
if(count ne 1) then begin
	MSP_BDP_print_text, Event, "ERROR - no image match for header data for image "+FILE_BASENAME(image)
endif else begin
	k = ind_this[0]
	(*n_ptr).image_HWP_angle[k] = new_ang
endelse
;
;	write revised header back to file
;
FXADDPAR, this_header, 'HISTORY', 'PPOL-Step 12-version '+(*s_ptr).version_short_string
writefits, image, image_data, this_header
;
no_write:
;
end