function bdp_model_background, input_image, SKY=sky
;
;	computes a smooth model of the background of an
;	image, and subtracts that from the
;	original image (putting back the mean sky)
;	to better reveal stars
;
;	DPC	20090701 	original code
;		20091008	added SKY keyword to return the sky value
;
;--------------------------------------------------
;
;	parses image into a grid of zones, and does mfm
;	for each zone, then fits a surface
;
im_size = SIZE(input_image)
xsize = im_size[1]
ysize = im_size[2]
;
;	if larger than one image, compute buffer size around edges
;
if(xsize gt 1024) then begin
	xborder = (xsize - 1024) / 2.0 + 5
endif else begin
	xborder = 5
endelse
;
if(ysize gt 1026) then begin
	yborder = (ysize - 1026) / 2.0 + 5
endif else begin
	yborder = 11
endelse
;
step = 30.0
;
;	how many zones? (ideally each zone would be 30x30 pixels
;			with adequate gaps between zones, of ~30 pixels)
;
nzone_x = FLOOR(FIX((xsize - 2*xborder) / step) / 2.0)
nzone_y = FLOOR(FIX((ysize - 2*yborder)/ step) / 2.0)
ngap_x = FLOOR(((xsize - 2*xborder) - nzone_x * step) / float(nzone_x - 1))
ngap_y = FLOOR(((ysize -2*yborder) - nzone_y * step) / float(nzone_y - 1))
x = indgen(nzone_x)
y = indgen(nzone_y)
;
;print,'xsize, ysize = ',xsize, ysize
;print, 'nzone_x, nzone_y = ',nzone_x, nzone_y
;print, 'ngap_x, ngap_y = ',ngap_x, ngap_y
;stop

;
grid = fltarr(3, nzone_x * nzone_y)
raw_grid = fltarr(nzone_x, nzone_y)
x_grid = fltarr(nzone_x, nzone_y)
y_grid = fltarr(nzone_x, nzone_y)
;
for ix = 0, nzone_x-1 do begin
	x0 = FLOOR(xborder + ix * (step + ngap_x) + step / 2.)
	x1 = (x0 + step-1) < (xsize - xborder)
	mean_x = float(x1 + x0)/2.0
	for iy = 0, nzone_y-1 do begin
		y0 = FLOOR(yborder + iy * (step + ngap_y) + step / 2.)
		y1 = (y0 + step-1) < (ysize - yborder)
		mean_y = float(y0 + y1) / 2.0
		;
		subarray = input_image[x0:x1,y0:y1]
		vec = REFORM(subarray[*,*])
		zz = median_filtered_mean(vec)
		;
;		print, "ix, iy, zz = ", ix, iy, zz[0], zz[1]
;		print, "x0, x1, y0, y1 = ",x0,x1,y0,y1
		;
		ixx = ix * nzone_y + iy
		grid[0,ixx] = mean_x
		grid[1,ixx] = mean_y
		grid[2,ixx] = zz[0]
		;
		raw_grid[ix, iy] = zz[0]
		x_grid[ix, iy] = mean_x
		y_grid[ix, iy] = mean_y
	endfor
endfor
;
;	remove 0;s and other big deviations
;
zz = median_filtered_mean(REFORM(grid[2,*]))
mean_sky = zz[0]
;
ind = WHERE(abs(grid[2,*] - zz[0])/zz[1] le 3.5,ngood)
if(ngood gt 0) then begin
	grid = grid[*,ind]
endif
;
;stop
;
;	grid computed, fit to surface
;
degree = ((nzone_x > nzone_y) - 4) < 4
Result = SFIT( grid, degree, KX=coeffs, /IRREGULAR, /MAX_DEGREE )
;print, 'coeffs = ',coeffs
;
;surface, raw_grid, x_grid, y_grid, charsize=2
;cursor, xx, yy, /wait
;wait, 0.5
;
;	use coefficients to compute surface
;
;new_surface = fltarr(nzone_x, nzone_y)
;for ix = 0, nzone_x-1 do begin
;	for iy = 0, nzone_y-1 do begin
;		this_x = x_grid[ix,iy]
;		this_y = y_grid[ix,iy]
;		;
;		new_surface[ix,iy] = mean_sky + coeffs[0] + this_y * coeffs[1] + coeffs[2] * this_y * this_y + coeffs[3] * this_y^3 + $
;							 		coeffs[4] * this_y^4 + $
;							 this_x * coeffs[5] + this_x * this_y * coeffs[6] + this_x * this_y * this_y * coeffs[7] + $
;							 		this_x * this_y * this_y * this_y * coeffs[8] + $
;							 this_x * this_x * coeffs[9] + this_x * this_x * this_y * coeffs[10] + $
;							 		this_x * this_x * this_y * this_y * coeffs[11] + $
;							 this_x * this_x * this_x * coeffs[12] + this_x * this_x * this_x * this_y * coeffs[13] + $
;							 this_x * this_x * this_x * this_x * coeffs[14]
;	endfor
;endfor
;surface, new_surface, x_grid, y_grid, charsize = 2
;cursor, xx, yy, /wait
;wait, 0.5
;
;	difference
;
;surface, raw_grid - new_surface, x_grid, y_grid, charsize=2
;cursor, xx, yy, /wait
;wait, 0.5
;
;	compute output image
;
output_image = input_image * 0.0
;
for ix = 0, xsize-1 do begin
	this_x = float(ix)
	for iy = 0, ysize-1 do begin
		this_y = float(iy)
		output_image[ix,iy] = coeffs[0] + this_y * coeffs[1] + coeffs[2] * this_y * this_y + coeffs[3] * this_y^3 + $
							 		coeffs[4] * this_y^4 + $
							 this_x * coeffs[5] + this_x * this_y * coeffs[6] + this_x * this_y * this_y * coeffs[7] + $
							 		this_x * this_y * this_y * this_y * coeffs[8] + $
							 this_x * this_x * coeffs[9] + this_x * this_x * this_y * coeffs[10] + $
							 		this_x * this_x * this_y * this_y * coeffs[11] + $
							 this_x * this_x * this_x * coeffs[12] + this_x * this_x * this_x * this_y * coeffs[13] + $
							 this_x * this_x * this_x * this_x * coeffs[14]
	endfor
endfor
;
;	compute background subtracted image, but put back the mean sky afterward
;
ind_bad_pix = WHERE(input_image lt -1.0e5,nbad)
;
output_image = input_image - output_image
output_image = output_image + mean_sky
if(nbad gt 0) then output_image[ind_bad_pix] = -1.0e6
;
sky = mean_sky
return, output_image
end
