pro coadd_median_with_bad_RAW1_sub_old, Event, bpf, pathlist, outpath, $
									row513_flag, LED_wid, this_LED, colors, $
									NO_OUTPUT=no_output, COADDS=coadds
	;
	;   adds (averages) Mimir images, from list of file pathnames supplied
	;   using a median filter for each pixel, rejecting
	;   outliers, if possible
	;
	;   this version does the RAW1 images
	;
	;   DPC 20050611 modified from coadd_median_with_bad.pro to work as a subroutine
	;       20051012 fixed file echo bug (thanks Frannie!)
	;       20051123 modified to include relative scaling by image means (median filtered), and
	;                    added pixel deletion conditions
	;       20051210 modified to trap incoming and outgoing pixels with values *below* bad_pix_value
	;		20130920 modified to correct for image coadds number on input read
	;
	;-----------------------------------------
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	;
	;	check status of no_output switch
	;
	if(KEYWORD_SET(no_output)) then NO_OUTPUT=1 else NO_OUTPUT=0
	if(N_ELEMENTS(COADDS) ne 0) then this_coadds = coadds else this_coadds = 1
	;
	image = read_raw1(pathlist[0])
	;
	if(this_coadds ne 1) then image /= this_coadds
	;
	i_size = SIZE(image)
	ncol = i_size[1]
	nrow = i_size[2]
	nimage = N_ELEMENTS(pathlist)
	;
	bad_pix_value = 0.0
	;
	;   load the bad pixel mask
	;
	bad_image = readfits(bpf,header,/silent)
	;
	;   read in each file into big arrays, using pointers
	;
	image_data_ptr = PTR_NEW(fltarr(nimage,ncol,nrow))
	accum_ptr = PTR_NEW(fltarr(ncol,nrow))
	;
	ngood = 0
	path = ''
	for ifile = 0L, nimage-1 do begin
		image = read_raw1(pathlist[ifile])
		;
		if(this_coadds ne 1) then image /= this_coadds
		;
    	(*image_data_ptr)[ifile,*,*] = float(image)
	endfor
	;
	colors[this_LED] = 'orange'
	WCT_LED_stack, LED_wid, colors
	;
	MSP_BDP_print_text, Event, "Read "+STRING(format='(i)',nimage)+" input RAW1 images"
	;
	;   do median filtered mean calculation with bad pixel masking
	;
	;   first, get means for each image
	;
	image_mean = dblarr(nimage)
	;
	for ifile = 0, nimage-1 do begin
		;
		;	subsample by 10x10 to speed things along
		;
	    image_single = REFORM((*image_data_ptr)[ifile,0:*:10,0:*:10])
	    im_size = SIZE(image_single)
	    nc = im_size[1]
	    nr = im_size[2]
	    image_single_vec = REFORM(image_single, long(nc)*long(nr))
	    image_vec = image_single_vec[WHERE(image_single_vec gt -10000. and image_single_vec lt 100000., icount)]
	    if(icount ge 100) then begin
	    	zz = median_filtered_mean(image_vec)
	    endif else begin
	    	zz = [0.,100.]
	    endelse
	    image_mean[ifile] = zz[0]
	endfor
	;
	colors[this_LED] = 'red'
	WCT_LED_stack, LED_wid, colors
	;
	;   compute relative scale factors -- we effectively scale to means before combining
	;
	zz = median_filtered_mean(image_mean)
	set_mean = zz[0]
	scale_factor = dblarr(nimage)
	scale_factor = image_mean / set_mean
	;
	ind500 = WHERE(image_mean gt 500, count_500)
	nimage_half = ROUND(nimage/2.0)
	;
	;   the pixel evaluation and combining engine
	;
	icolor = 'blue'
	for icol = 0L, ncol-1L do begin
		;
		icol25 = round(icol/25L)*25L
		if(icol25 eq icol) then begin
			colors[this_LED] = icolor
			WCT_LED_stack, LED_wid, colors
			if(icolor eq 'blue') then icolor = 'violet' else icolor = 'blue'
		endif
		;
	    for irow = 0, nrow-1 do begin
	       if(bad_image[icol,irow] ne 1) then begin

				pix_vec = REFORM( (*image_data_ptr)[*,icol,irow] )
	       		;
	       		ind_good = WHERE(pix_vec ne bad_pix_value, count_good)
	       		if(count_good ge nimage_half) then begin
	       			pix_vec = pix_vec[ind_good]	* scale_factor[ind_good]	; strip deleted pixels, scale others
	       			;
					if(count_500 gt 0) then begin
						pix_frac = pix_vec / image_mean[ind_good]
						ind_high = WHERE(pix_frac gt 0.1, count_high)
						if(count_high gt 0) then begin
							pix_vec = pix_vec[ind_high]
							;
							;	passed all but row513 test, so go ahead and calc
							;
			                mx = median_filtered_mean(pix_vec)
		                    ;
		                    (*accum_ptr)[icol,irow] = mx[0]     ; the final mean value
		                    ;
                 		endif else (*accum_ptr)[icol,irow] = bad_pix_value
			        endif else begin
			        	mx = median_filtered_mean(pix_vec[ind_good])
	                    ;
	                    (*accum_ptr)[icol,irow] = mx[0]     ; the final mean value
	                endelse
	             endif else (*accum_ptr)[icol,irow] = bad_pix_value
	       endif else begin
	         (*accum_ptr)[icol,irow] = bad_pix_value     ; bad pixel value
	       endelse
	    endfor
	endfor
	;
	;   all pixels processed, save
	;
	;   trap bad pixels below bad_pix_value
	;
	ind = WHERE( (*accum_ptr) lt bad_pix_value,count)
	if(count gt 0) then (*accum_ptr)[ind] = bad_pix_value
	;
	if(NO_OUTPUT ne 1) then begin
		ier = write_raw1(outpath,(*accum_ptr) )
		;
		MSP_BDP_print_text,Event, "Wrote Coadded Image "+outpath+" to disk"
	endif
	;
	;	release the pointers
	;
	PTR_FREE, accum_ptr, image_data_ptr
	;
end