function kokepeli_offset_fitter, gfit, debug
;
;   fits a line to the run of kokepeli crack locations vs column
;   for each of the four quads
;
;   this version assumes a slope of 1.759259 for all RH cracks
;
;   examines residuals, rejects outliers, refits, loops
;
;   DPC 20050622 initial code borrowed from kokepeli_line_fitter.pro
;       20050726 modified for offset fitting with constant slope
;
;   gfit comes in as (4,3,21) = (quad, row-value/row-uncertainty/column-value, column index)
;
;   out_fit return array goes out as (4,2,2) = (quad, slope/offset, value/uncertainty)
;
;----------------------------------------------------
;
out_fit = fltarr(4,2,2)
;
;   for each quad
;
for iquad = 0, 3 do begin
    ;
    x = fltarr(21)
    x = reform(gfit[iquad,2,*])
    y = fltarr(21)
    y = reform(gfit[iquad,0,*])
    sy = fltarr(21)
    sy = reform(gfit[iquad,1,*])
    ind = where(x gt -10)
    ;
    ;
    Result = offset_fit(x,y,MEASURE_ERRORS=sy,SIGMA=sfit,YFIT=yfit)
    ;
    ;   report fit, uncertainty
    ;
    if(debug ne 0) then begin
        print, "Quad ",iquad," Slope = ",Result[1]," +/- ",sfit[1]
        print, "              Offset = ",Result[0]," +/- ",sfit[0]
    endif
    ;
    ;   compute residuals
    ;
    resid = y - yfit
    yfit_old = yfit
    ;
    ; ********** do cleaning, refitting here *****
    ;
    sum_old=21
    for iloop = 0, 4 do begin
        ;
        ;  restart resids with no deletions to prevent collapse of sample set
        ;
        resid = y - yfit_old
        ;
        mean = median_filtered_mean(resid)
        ;
        if(debug ne 0) then print, "Residuals, Mean = ",mean[0]," rms = ",mean[1]
        ;
        ind = where(abs(resid-mean[0]) lt (3.5+0.2*iloop)*mean[1])
        sum = n_elements(where(ind ne 0))
        ;
        if(debug ne 0) then print, "Sum = ",sum," sum_old = ",sum_old
        ;
        if (sum lt sum_old) then begin
           sum_old = sum
           ;
           if(debug ne 0) then print, "Refit, loop = ",iloop
           ;
           xi = x[ind]
           yi = y[ind]
           syi = sy[ind]
           Result = offset_fit(xi,yi,MEASURE_ERRORS=syi,SIGMA=sfit,YFIT=yfit)
           ;
           ;   report fit, uncertainty
           ;
           if(debug ne 0) then begin
                print, "Refit: Quad ",iquad," Slope = ",Result[1]," +/- ",sfit[1]
                print, "Refit:               Offset = ",Result[0]," +/- ",sfit[0]
           endif
           ;
           ;   compute residuals
           ;
           resid = y[ind] - yfit
           n = n_elements(ind)
           resid = resid[0:n-1]
           ;
        endif else goto, zout
    endfor
    ;----------------------------------------------------------------------------
    zout: zz = 0
    ;
    ;   plot
    ;
    if(debug ne 0) then begin
        window, 11
        plot, x[ind], resid[ind], yrange=[-1,1],psym=4
        sy10 = sy[ind]
        ploterr, x[ind],resid[ind], sy10, 3
        ;
        ;print, "x,y,errors"
        ;print, x[ind]
        ;print, y[ind]
        print, sy[ind]

        cursor, xx, yy, /wait
        wait, 0.5
        wdelete, 11
    endif
    ;
    ;   save results
    out_fit[iquad, 0, 0] = Result[1]
    out_fit[iquad, 0, 1] = sfit[1]
    out_fit[iquad, 1, 0] = Result[0]
    out_fit[iquad, 1, 1] = sfit[0]
    ;
endfor
;
;   done
;
return, out_fit
end

