pro linearity_correct, Event, input_CDS_path, input_RAW1_path, average_RAW1_bias, bad_pixel_mask, $
						corrected_image, corrected_header, coadds
;
;   performs linearity correction on CDS images
;
;   checks for bad pixels, then saturation, then does correction
;   if neither true. If either bad pix or saturation, sets pixel
;   value to -1.0e6
;
;   DPC 20070407    initial code
;       20041030    modified for Oct '04 data
;       20050105    modified for Dec '04 data
;       20050624    modified for (Reset-RAW1,2) correction method
;       20050711    modified to include RAW1 directory selection
;       20050714    modified to use new 9-image LC tables
;       20050925    tuned linearize.pro
;       20050925    added output directory creator *** need to compute RAW1 directory rather than input it...*
;       20051026    modified to recognize OS type
;       20051117    added NOGUI switch
;       20051118    modified to allow selection of either files or file of filenames
;       20060129    modified to work with subframed images
;       20060627    fixed bug on input list for batch runs
;		20071223	incorporated into MSP_BDP, stripped to work one file at a time, with linearity tables in memory
;		20130920	added coadds to scale down input image; but then rescale back to sum on exit
;
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
s_ptr = PTR_struct.PTR_summary
l_ptr = PTR_struct.PTR_Linearity
lc_ptr = PTR_struct.PTR_LC
n_ptr = PTR_struct.PTR_night
b_ptr = PTR_struct.PTR_BPM
a_ptr = PTR_struct.PTR_Ave_RAW1
;
itest = 1L
;
;   order of LC correction in tables
;
norder = 4
nlcfiles = 2*norder+1     ; number of LC image files
;
bad_value = -1.0e6
bad_band = abs(bad_value)/20.
sat_value = -1.0e6
;
bad_pix_image = readfits(bad_pixel_mask, bad_header,/silent)
;
;   establish size of images
;
image_size = SIZE(bad_pix_image)
ncol = image_size[1]
nrow = image_size[2]
;
;   had to call bad pix first, since nc x nr format
;   embedded in it, but not in RAW1 image
;
bias = readfits(average_RAW1_bias, bias_header, /silent)	; no scaling if coadds; already AVERAGE type
;
;   linearity data
;
lin_data = (*(*l_ptr).lin_table_ptr)[*,*,*]
;
;   2. Set up, then linearize (Reset-RAW1) image
;
;     read CDS image, read RAW1 image
;
if (FILE_TEST(input_CDS_path)) then begin
	image_CDS = double(readfits(input_CDS_path,corrected_header,/silent))
;	if(coadds ne 1) then image_CDS /= float(coadds)
endif else $
	MSP_BDP_print_text, Event, "Error opening file "+input_CDS_path
if (FILE_TEST(input_RAW1_path)) then begin
	image_RAW1 = double(read_raw1(input_RAW1_path,ncol,nrow))
;	if(coadds ne 1) then image_RAW1 /= coadds
endif else $
	MSP_BDP_print_text, Event, "Error opening file "+input_RAW1_path
;
;     check for and correct for COADDS (actual number, not requested)
;
;coadds = FXPAR(corrected_header, 'NUMCOADD',COUNT=count)
;if (count lt 1 or coadds eq 0) then begin
;    coadds = 1
;endif
;
if(coadds ne 1) then MSP_BDP_print_text,Event, "Coadds = "+STRING(format='(i5.1)',coadds)
;
bpi = bad_pix_image
;image_CDS = image_CDS / double(coadds)     ;!!!!!!!!!!!!!
if(coadds ne 1) then begin
	image_CDS = safe_math(image_CDS, coadds, bpi, '/')		; convert SUM to AVERAGE
	image_RAW1 = safe_math(image_RAW1, coadds, bpi, '/')	; same
endif
;
;     compute (Reset-RAW1)
;
;RR1 = bias - image_RAW1          ;!!!!!!!!!!!!!; *more* signal is lower ADUs in individual frames, so this difference gives positive signals
bpi = bad_pix_image
RR1 = safe_math(bias, image_RAW1, bpi, '-')
;
;     linearize this
;
lc_data = lin_data
bpi = bad_pix_image
nnorder = norder
RRLC1 = linearize(RR1, bpi, lc_data, nnorder)
;
;  3. Set up, then linearize (Reset-RAW2) image
;
;RR2 = image_CDS + RR1       ;!!!!!!!!!!!!!
bpi = bad_pix_image
RR2 = safe_math(image_CDS, RR1, bpi, '+')
lc_data = lin_data
nnorder = norder
bpi = bad_pix_image
RRLC2 = linearize(RR2, bpi, lc_data, nnorder)
;
;  4. Compute new, corrected CDS image [(Reset-RAW2) - (Reset-RAW1)]
;
bpi = bad_pix_image
CDS = safe_math(RRLC2, RRLC1, bpi, '-')
bpi = bad_pix_image
;
if(coadds ne 1) then CDS = safe_math(CDS, coadds, bpi, "*")	; multiply by coadds - to produce SUM values in pixels
;
;   reduce precision back to single to conserve disk space  (20050906)
;
corrected_image = float(CDS)
;
end

