function linearize_new, input_image, bad_pix_image, lin_data, norder
;
;   linearizer engine
;
;   assumes input "image" is of form "Reset - RAW1" or "Reset - RAW2"
;   and so applicable by linearity files
;
;   DPC 20050624 initial code
;       20050714 modified to utilize order of LC corrections
;       20050715 modified to set zero saturation pixels to bad pix value
;       20050924 modified conditions for setting bad pixel value
;       20050925 continued tuning of rejection parameters
;		20080117 sped up by vectorizing more
;
;
bad_pix_value = -1.0e6
;
image_size = SIZE(input_image)
;
nrow = image_size[2]
ncol = image_size[1]
;
;lin_size = SIZE(REFORM(lin_data[0,*,*]))
;lin_col = lin_size[1]
;lin_row = lin_size[2]
;
output_image = fltarr(ncol, nrow)
;
;   map LC images to correction terms for different order numbers
;
case norder of
    2: begin
         cc2 = REFORM(lin_data[1,*,*])
         cc3 = cc2 * 0.0
         cc4 = cc2 * 0.0
         t0 = REFORM(lin_data[2,*,*])
         t1 = REFORM(lin_data[3,*,*])
         t2 = REFORM(lin_data[4,*,*])
         t3 = t2 * 0.0
         t4 = t2 * 0.0
       end
    3: begin
         cc2 = REFORM(lin_data[1,*,*])
         cc3 = REFORM(lin_data[2,*,*])
         cc4 = cc2 * 0.0
         t0 = REFORM(lin_data[3,*,*])
         t1 = REFORM(lin_data[4,*,*])
         t2 = REFORM(lin_data[5,*,*])
         t3 = REFORM(lin_data[6,*,*])
         t4 = t2 * 0.0
       end
    4: begin
         cc2 = REFORM(lin_data[1,*,*])
         cc3 = REFORM(lin_data[2,*,*])
         cc4 = REFORM(lin_data[3,*,*])
         t0 = REFORM(lin_data[4,*,*])
         t1 = REFORM(lin_data[5,*,*])
         t2 = REFORM(lin_data[6,*,*])
         t3 = REFORM(lin_data[7,*,*])
         t4 = REFORM(lin_data[8,*,*])
       end
endcase
;
test_row = 224
;
for irow = 0, nrow-1 do begin
	;
	bad_vec = REFORM(bad_pix_image[*,irow])
	input_vec = REFORM(input_image[*,irow])
	;
	ind_bad = WHERE(bad_vec, bad_count)
	ind_a_bad = WHERE(input_vec eq bad_pix_value, bad_a_count)
	;
	sat_vec = REFORM(lin_data[0,*,irow])
	ind_sat = WHERE(sat_vec le input_vec or sat_vec eq bad_pix_value, bad_sat_count)
	;
;	if(irow eq test_row) then begin
;		print,"BPM pixels in row = ",bad_count
;		if(bad_count ne 0) then print,"  indices = ",ind_bad
;		print, "Deleted input pixels in row = ",bad_a_count
;		if(bad_a_count ne 0) then print, "   indices = ",ind_a_bad
;		print, "Saturate pixels in row = ",bad_sat_count
;		if(bad_sat_count ne 0) then print, "  indices = ",ind_sat
;	endif
	;
	;	collect all the bad indices
	;
	ind_all = 0L
	n_ind = 0L
	if(bad_count ne 0) then begin
		ind_all = ind_bad
		n_ind = bad_count
	endif
	if(bad_a_count ne 0) then begin
		if(n_ind eq 0) then begin
			ind_all = ind_a_bad
			n_ind = bad_a_count
		endif else begin
			ind_all = [ind_all, ind_a_bad]
			n_ind = n_ind + bad_a_count
		endelse
	endif
	if(bad_sat_count ne 0) then begin
		if(n_ind eq 0) then begin
			ind_all = ind_sat
			n_ind = bad_sat_count
		endif else begin
			ind_all = [ind_all, ind_sat]
			n_ind = n_ind + bad_sat_count
		endelse
	endif
	if(n_ind ne 0) then begin
		ind_sort = SORT(ind_all)
		ind_uniq = UNIQ(ind_all[ind_sort])
		ind_uniq = ind_all[ind_sort[ind_uniq]]
		n_ind = N_ELEMENTS(ind_uniq)
		;
;;		if(irow eq test_row) then print, "row ",irow," deleted indices - ",ind_uniq
	endif
	;
	;	vectorize coefficients
	;
	t_0_vec = REFORM(t0[*,irow])
	t_1_vec = REFORM(t1[*,irow])
	t_2_vec = REFORM(t2[*,irow])
	t_3_vec = REFORM(t3[*,irow])
	t_4_vec = REFORM(t4[*,irow])
	;
	cc_2_vec = REFORM(cc2[*,irow])
	cc_3_vec = REFORM(cc3[*,irow])
	cc_4_vec = REFORM(cc4[*,irow])
	;
	;	compute the effective time vector
	;
	t_vec = t_0_vec + t_1_vec * input_vec + t_2_vec * (input_vec^2) + $
			t_3_vec * (input_vec^3) + t_4_vec*(input_vec^4)
	;
	;	compute correction vector
	;
    corr_vec = -1.0d0 * (t_vec * t_vec * cc_2_vec + (t_vec^3)*cc_3_vec + (t_vec^4) * cc_4_vec)
    ;
    ;	compute corrected output vector
    ;
    output_vec = input_vec + corr_vec
    ;
    ;	and then perform all the tests
    ;
    ind_low = WHERE(corr_vec lt 0.0 and input_vec ne 0.0, low_count)
    if(low_count ne 0) then begin
    	;
    	; correction negative
    	;
   		; 	if correction more neg than -20, zero correction
   		;
   		ind_20 = WHERE(corr_vec[ind_low] lt -20.0,low_20_count)
   		if(low_20_count ne 0) then begin
   			output_vec[ind_low[ind_20]] = input_vec[ind_low[ind_20]]	; clear correction for this case
   		endif
   		;
   		;	if input pixel more negative than -300, delete the output pixel
   		;
   		ind_inv = WHERE(input_vec[ind_low] lt -300.0, count_low_inv)
   		if(count_low_inv ne 0) then begin
   			output_vec[ind_low[ind_inv]] = bad_pix_value
   		endif
   		;
   		;	if correction within [-20, 0] and input value more positive than -300.0, clear correction
   		;
   		ind_z = WHERE(input_vec[ind_low] gt -300.0 and corr_vec[ind_low] ge -20.0, count_z)
   		if(count_z ne 0) then begin
   			output_vec[ind_low[ind_z]] = input_vec[ind_low[ind_z]]
   		endif
   	endif
   	;
   	output_vec[ind_uniq] = bad_pix_value
    ;
   	output_image[*,irow] = float(output_vec)
   	;
endfor
;
return, output_image
;
end